@@ -83,13 +83,15 @@ struct activity cpu_act = {
 	.f_print	= print_cpu_stats,
 	.f_print_avg	= print_cpu_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "CPU;%user;%nice;%system;%iowait;%steal;%idle|"
+		          "CPU;%usr;%nice;%sys;%iowait;%steal;%irq;%soft;%guest;%gnice;%idle",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_cpu_stats,
 	.f_xml_print	= xml_print_cpu_stats,
 	.f_json_print	= json_print_cpu_stats,
 	.f_svg_print	= svg_print_cpu_stats,
-	.hdr_line	= "CPU;%user;%nice;%system;%iowait;%steal;%idle|"
-		          "CPU;%usr;%nice;%sys;%iowait;%steal;%irq;%soft;%guest;%gnice;%idle",
 	.name		= "A_CPU",
 	.g_nr		= 1,
 #endif
@@ -118,12 +120,14 @@ struct activity pcsw_act = {
 	.f_print	= print_pcsw_stats,
 	.f_print_avg	= print_pcsw_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "proc/s;cswch/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_pcsw_stats,
 	.f_xml_print	= xml_print_pcsw_stats,
 	.f_json_print	= json_print_pcsw_stats,
 	.f_svg_print	= svg_print_pcsw_stats,
-	.hdr_line	= "proc/s;cswch/s",
 	.name		= "A_PCSW",
 	.g_nr		= 2,
 #endif
@@ -152,12 +156,13 @@ struct activity irq_act = {
 	.f_print	= print_irq_stats,
 	.f_print_avg	= print_irq_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "INTR;intr/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_irq_stats,
 	.f_xml_print	= xml_print_irq_stats,
 	.f_json_print	= json_print_irq_stats,
-	.f_svg_print	= NULL,
-	.hdr_line	= "INTR;intr/s",
 	.name		= "A_IRQ",
 	.g_nr		= 0,
 #endif
@@ -186,12 +191,14 @@ struct activity swap_act = {
 	.f_print	= print_swap_stats,
 	.f_print_avg	= print_swap_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "pswpin/s;pswpout/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_swap_stats,
 	.f_xml_print	= xml_print_swap_stats,
 	.f_json_print	= json_print_swap_stats,
 	.f_svg_print	= svg_print_swap_stats,
-	.hdr_line	= "pswpin/s;pswpout/s",
 	.name		= "A_SWAP",
 	.g_nr		= 1,
 #endif
@@ -220,13 +227,15 @@ struct activity paging_act = {
 	.f_print	= print_paging_stats,
 	.f_print_avg	= print_paging_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "pgpgin/s;pgpgout/s;fault/s;majflt/s;"
+		          "pgfree/s;pgscank/s;pgscand/s;pgsteal/s;%vmeff",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_paging_stats,
 	.f_xml_print	= xml_print_paging_stats,
 	.f_json_print	= json_print_paging_stats,
 	.f_svg_print	= svg_print_paging_stats,
-	.hdr_line	= "pgpgin/s;pgpgout/s;fault/s;majflt/s;"
-		          "pgfree/s;pgscank/s;pgscand/s;pgsteal/s;%vmeff",
 	.name		= "A_PAGE",
 	.g_nr		= 3,
 #endif
@@ -255,12 +264,14 @@ struct activity io_act = {
 	.f_print	= print_io_stats,
 	.f_print_avg	= print_io_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "tps;rtps;wtps;bread/s;bwrtn/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_io_stats,
 	.f_xml_print	= xml_print_io_stats,
 	.f_json_print	= json_print_io_stats,
 	.f_svg_print	= svg_print_io_stats,
-	.hdr_line	= "tps;rtps;wtps;bread/s;bwrtn/s",
 	.name		= "A_IO",
 	.g_nr		= 2,
 #endif
@@ -289,14 +300,16 @@ struct activity memory_act = {
 	.f_print	= print_memory_stats,
 	.f_print_avg	= print_avg_memory_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "frmpg/s;bufpg/s;campg/s|"
+		          "kbmemfree;kbmemused;%memused;kbbuffers;kbcached;kbcommit;%commit;kbactive;kbinact;kbdirty&kbanonpg;kbslab;kbkstack;kbpgtbl;kbvmused|"
+		          "kbswpfree;kbswpused;%swpused;kbswpcad;%swpcad",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_memory_stats,
 	.f_xml_print	= xml_print_memory_stats,
 	.f_json_print	= json_print_memory_stats,
 	.f_svg_print	= svg_print_memory_stats,
-	.hdr_line	= "frmpg/s;bufpg/s;campg/s|"
-		          "kbmemfree;kbmemused;%memused;kbbuffers;kbcached;kbcommit;%commit;kbactive;kbinact;kbdirty&kbanonpg;kbslab;kbkstack;kbpgtbl;kbvmused|"
-		          "kbswpfree;kbswpused;%swpused;kbswpcad;%swpcad",
 	.name		= "A_MEMORY",
 	.g_nr		= 9,
 #endif
@@ -325,12 +338,14 @@ struct activity ktables_act = {
 	.f_print	= print_ktables_stats,
 	.f_print_avg	= print_avg_ktables_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "dentunusd;file-nr;inode-nr;pty-nr",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_ktables_stats,
 	.f_xml_print	= xml_print_ktables_stats,
 	.f_json_print	= json_print_ktables_stats,
 	.f_svg_print	= svg_print_ktables_stats,
-	.hdr_line	= "dentunusd;file-nr;inode-nr;pty-nr",
 	.name		= "A_KTABLES",
 	.g_nr		= 2,
 #endif
@@ -359,12 +374,14 @@ struct activity queue_act = {
 	.f_print	= print_queue_stats,
 	.f_print_avg	= print_avg_queue_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "runq-sz;plist-sz;ldavg-1;ldavg-5;ldavg-15;blocked",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_queue_stats,
 	.f_xml_print	= xml_print_queue_stats,
 	.f_json_print	= json_print_queue_stats,
 	.f_svg_print	= svg_print_queue_stats,
-	.hdr_line	= "runq-sz;plist-sz;ldavg-1;ldavg-5;ldavg-15;blocked",
 	.name		= "A_QUEUE",
 	.g_nr		= 3,
 #endif
@@ -393,12 +410,13 @@ struct activity serial_act = {
 	.f_print	= print_serial_stats,
 	.f_print_avg	= print_serial_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "TTY;rcvin/s;txmtin/s;framerr/s;prtyerr/s;brk/s;ovrun/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_serial_stats,
 	.f_xml_print	= xml_print_serial_stats,
 	.f_json_print	= json_print_serial_stats,
-	.f_svg_print	= NULL,
-	.hdr_line	= "TTY;rcvin/s;txmtin/s;framerr/s;prtyerr/s;brk/s;ovrun/s",
 	.name		= "A_SERIAL",
 	.g_nr		= 0,
 #endif
@@ -427,12 +445,14 @@ struct activity disk_act = {
 	.f_print	= print_disk_stats,
 	.f_print_avg	= print_disk_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "DEV;tps;rd_sec/s;wr_sec/s;avgrq-sz;avgqu-sz;await;svctm;%util",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_disk_stats,
 	.f_xml_print	= xml_print_disk_stats,
 	.f_json_print	= json_print_disk_stats,
 	.f_svg_print	= svg_print_disk_stats,
-	.hdr_line	= "DEV;tps;rd_sec/s;wr_sec/s;avgrq-sz;avgqu-sz;await;svctm;%util",
 	.name		= "A_DISK",
 	.g_nr		= 5,
 #endif
@@ -461,12 +481,14 @@ struct activity net_dev_act = {
 	.f_print	= print_net_dev_stats,
 	.f_print_avg	= print_net_dev_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "IFACE;rxpck/s;txpck/s;rxkB/s;txkB/s;rxcmp/s;txcmp/s;rxmcst/s;%ifutil",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_dev_stats,
 	.f_xml_print	= xml_print_net_dev_stats,
 	.f_json_print	= json_print_net_dev_stats,
 	.f_svg_print	= svg_print_net_dev_stats,
-	.hdr_line	= "IFACE;rxpck/s;txpck/s;rxkB/s;txkB/s;rxcmp/s;txcmp/s;rxmcst/s;%ifutil",
 	.name		= "A_NET_DEV",
 	.g_nr		= 4,
 #endif
@@ -495,13 +517,15 @@ struct activity net_edev_act = {
 	.f_print	= print_net_edev_stats,
 	.f_print_avg	= print_net_edev_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "IFACE;rxerr/s;txerr/s;coll/s;rxdrop/s;txdrop/s;"
+		          "txcarr/s;rxfram/s;rxfifo/s;txfifo/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_edev_stats,
 	.f_xml_print	= xml_print_net_edev_stats,
 	.f_json_print	= json_print_net_edev_stats,
 	.f_svg_print	= svg_print_net_edev_stats,
-	.hdr_line	= "IFACE;rxerr/s;txerr/s;coll/s;rxdrop/s;txdrop/s;"
-		          "txcarr/s;rxfram/s;rxfifo/s;txfifo/s",
 	.name		= "A_NET_EDEV",
 	.g_nr		= 4,
 #endif
@@ -530,12 +554,14 @@ struct activity net_nfs_act = {
 	.f_print	= print_net_nfs_stats,
 	.f_print_avg	= print_net_nfs_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "call/s;retrans/s;read/s;write/s;access/s;getatt/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_nfs_stats,
 	.f_xml_print	= xml_print_net_nfs_stats,
 	.f_json_print	= json_print_net_nfs_stats,
 	.f_svg_print	= svg_print_net_nfs_stats,
-	.hdr_line	= "call/s;retrans/s;read/s;write/s;access/s;getatt/s",
 	.name		= "A_NET_NFS",
 	.g_nr		= 3,
 #endif
@@ -564,13 +590,15 @@ struct activity net_nfsd_act = {
 	.f_print	= print_net_nfsd_stats,
 	.f_print_avg	= print_net_nfsd_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "scall/s;badcall/s;packet/s;udp/s;tcp/s;hit/s;miss/s;"
+		          "sread/s;swrite/s;saccess/s;sgetatt/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_nfsd_stats,
 	.f_xml_print	= xml_print_net_nfsd_stats,
 	.f_json_print	= json_print_net_nfsd_stats,
 	.f_svg_print	= svg_print_net_nfsd_stats,
-	.hdr_line	= "scall/s;badcall/s;packet/s;udp/s;tcp/s;hit/s;miss/s;"
-		          "sread/s;swrite/s;saccess/s;sgetatt/s",
 	.name		= "A_NET_NFSD",
 	.g_nr		= 5,
 #endif
@@ -599,12 +627,14 @@ struct activity net_sock_act = {
 	.f_print	= print_net_sock_stats,
 	.f_print_avg	= print_avg_net_sock_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "totsck;tcpsck;udpsck;rawsck;ip-frag;tcp-tw",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_sock_stats,
 	.f_xml_print	= xml_print_net_sock_stats,
 	.f_json_print	= json_print_net_sock_stats,
 	.f_svg_print	= svg_print_net_sock_stats,
-	.hdr_line	= "totsck;tcpsck;udpsck;rawsck;ip-frag;tcp-tw",
 	.name		= "A_NET_SOCK",
 	.g_nr		= 2,
 #endif
@@ -633,12 +663,14 @@ struct activity net_ip_act = {
 	.f_print	= print_net_ip_stats,
 	.f_print_avg	= print_net_ip_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "irec/s;fwddgm/s;idel/s;orq/s;asmrq/s;asmok/s;fragok/s;fragcrt/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_ip_stats,
 	.f_xml_print	= xml_print_net_ip_stats,
 	.f_json_print	= json_print_net_ip_stats,
 	.f_svg_print	= svg_print_net_ip_stats,
-	.hdr_line	= "irec/s;fwddgm/s;idel/s;orq/s;asmrq/s;asmok/s;fragok/s;fragcrt/s",
 	.name		= "A_NET_IP",
 	.g_nr		= 3,
 #endif
@@ -667,12 +699,14 @@ struct activity net_eip_act = {
 	.f_print	= print_net_eip_stats,
 	.f_print_avg	= print_net_eip_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "ihdrerr/s;iadrerr/s;iukwnpr/s;idisc/s;odisc/s;onort/s;asmf/s;fragf/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_eip_stats,
 	.f_xml_print	= xml_print_net_eip_stats,
 	.f_json_print	= json_print_net_eip_stats,
 	.f_svg_print	= svg_print_net_eip_stats,
-	.hdr_line	= "ihdrerr/s;iadrerr/s;iukwnpr/s;idisc/s;odisc/s;onort/s;asmf/s;fragf/s",
 	.name		= "A_NET_EIP",
 	.g_nr		= 3,
 #endif
@@ -701,13 +735,15 @@ struct activity net_icmp_act = {
 	.f_print	= print_net_icmp_stats,
 	.f_print_avg	= print_net_icmp_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "imsg/s;omsg/s;iech/s;iechr/s;oech/s;oechr/s;itm/s;itmr/s;otm/s;"
+		          "otmr/s;iadrmk/s;iadrmkr/s;oadrmk/s;oadrmkr/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_icmp_stats,
 	.f_xml_print	= xml_print_net_icmp_stats,
 	.f_json_print	= json_print_net_icmp_stats,
 	.f_svg_print	= svg_print_net_icmp_stats,
-	.hdr_line	= "imsg/s;omsg/s;iech/s;iechr/s;oech/s;oechr/s;itm/s;itmr/s;otm/s;"
-		          "otmr/s;iadrmk/s;iadrmkr/s;oadrmk/s;oadrmkr/s",
 	.name		= "A_NET_ICMP",
 	.g_nr		= 4,
 #endif
@@ -736,13 +772,15 @@ struct activity net_eicmp_act = {
 	.f_print	= print_net_eicmp_stats,
 	.f_print_avg	= print_net_eicmp_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "ierr/s;oerr/s;idstunr/s;odstunr/s;itmex/s;otmex/s;"
+		          "iparmpb/s;oparmpb/s;isrcq/s;osrcq/s;iredir/s;oredir/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_eicmp_stats,
 	.f_xml_print	= xml_print_net_eicmp_stats,
 	.f_json_print	= json_print_net_eicmp_stats,
 	.f_svg_print	= svg_print_net_eicmp_stats,
-	.hdr_line	= "ierr/s;oerr/s;idstunr/s;odstunr/s;itmex/s;otmex/s;"
-		          "iparmpb/s;oparmpb/s;isrcq/s;osrcq/s;iredir/s;oredir/s",
 	.name		= "A_NET_EICMP",
 	.g_nr		= 6,
 #endif
@@ -771,12 +809,14 @@ struct activity net_tcp_act = {
 	.f_print	= print_net_tcp_stats,
 	.f_print_avg	= print_net_tcp_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "active/s;passive/s;iseg/s;oseg/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_tcp_stats,
 	.f_xml_print	= xml_print_net_tcp_stats,
 	.f_json_print	= json_print_net_tcp_stats,
 	.f_svg_print	= svg_print_net_tcp_stats,
-	.hdr_line	= "active/s;passive/s;iseg/s;oseg/s",
 	.name		= "A_NET_TCP",
 	.g_nr		= 2,
 #endif
@@ -805,12 +845,14 @@ struct activity net_etcp_act = {
 	.f_print	= print_net_etcp_stats,
 	.f_print_avg	= print_net_etcp_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "atmptf/s;estres/s;retrans/s;isegerr/s;orsts/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_etcp_stats,
 	.f_xml_print	= xml_print_net_etcp_stats,
 	.f_json_print	= json_print_net_etcp_stats,
 	.f_svg_print	= svg_print_net_etcp_stats,
-	.hdr_line	= "atmptf/s;estres/s;retrans/s;isegerr/s;orsts/s",
 	.name		= "A_NET_ETCP",
 	.g_nr		= 2,
 #endif
@@ -839,12 +881,14 @@ struct activity net_udp_act = {
 	.f_print	= print_net_udp_stats,
 	.f_print_avg	= print_net_udp_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "idgm/s;odgm/s;noport/s;idgmerr/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_udp_stats,
 	.f_xml_print	= xml_print_net_udp_stats,
 	.f_json_print	= json_print_net_udp_stats,
 	.f_svg_print	= svg_print_net_udp_stats,
-	.hdr_line	= "idgm/s;odgm/s;noport/s;idgmerr/s",
 	.name		= "A_NET_UDP",
 	.g_nr		= 2,
 #endif
@@ -873,12 +917,14 @@ struct activity net_sock6_act = {
 	.f_print	= print_net_sock6_stats,
 	.f_print_avg	= print_avg_net_sock6_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "tcp6sck;udp6sck;raw6sck;ip6-frag",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_sock6_stats,
 	.f_xml_print	= xml_print_net_sock6_stats,
 	.f_json_print	= json_print_net_sock6_stats,
 	.f_svg_print	= svg_print_net_sock6_stats,
-	.hdr_line	= "tcp6sck;udp6sck;raw6sck;ip6-frag",
 	.name		= "A_NET_SOCK6",
 	.g_nr		= 1,
 #endif
@@ -907,13 +953,15 @@ struct activity net_ip6_act = {
 	.f_print	= print_net_ip6_stats,
 	.f_print_avg	= print_net_ip6_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "irec6/s;fwddgm6/s;idel6/s;orq6/s;asmrq6/s;asmok6/s;"
+			  "imcpck6/s;omcpck6/s;fragok6/s;fragcr6/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_ip6_stats,
 	.f_xml_print	= xml_print_net_ip6_stats,
 	.f_json_print	= json_print_net_ip6_stats,
 	.f_svg_print	= svg_print_net_ip6_stats,
-	.hdr_line	= "irec6/s;fwddgm6/s;idel6/s;orq6/s;asmrq6/s;asmok6/s;"
-			  "imcpck6/s;omcpck6/s;fragok6/s;fragcr6/s",
 	.name		= "A_NET_IP6",
 	.g_nr		= 4,
 #endif
@@ -942,13 +990,15 @@ struct activity net_eip6_act = {
 	.f_print	= print_net_eip6_stats,
 	.f_print_avg	= print_net_eip6_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "ihdrer6/s;iadrer6/s;iukwnp6/s;i2big6/s;idisc6/s;odisc6/s;"
+			  "inort6/s;onort6/s;asmf6/s;fragf6/s;itrpck6/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_eip6_stats,
 	.f_xml_print	= xml_print_net_eip6_stats,
 	.f_json_print	= json_print_net_eip6_stats,
 	.f_svg_print	= svg_print_net_eip6_stats,
-	.hdr_line	= "ihdrer6/s;iadrer6/s;iukwnp6/s;i2big6/s;idisc6/s;odisc6/s;"
-			  "inort6/s;onort6/s;asmf6/s;fragf6/s;itrpck6/s",
 	.name		= "A_NET_EIP6",
 	.g_nr		= 4,
 #endif
@@ -977,14 +1027,16 @@ struct activity net_icmp6_act = {
 	.f_print	= print_net_icmp6_stats,
 	.f_print_avg	= print_net_icmp6_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "imsg6/s;omsg6/s;iech6/s;iechr6/s;oechr6/s;igmbq6/s;igmbr6/s;ogmbr6/s;"
+			  "igmbrd6/s;ogmbrd6/s;irtsol6/s;ortsol6/s;irtad6/s;inbsol6/s;onbsol6/s;"
+			  "inbad6/s;onbad6/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_icmp6_stats,
 	.f_xml_print	= xml_print_net_icmp6_stats,
 	.f_json_print	= json_print_net_icmp6_stats,
 	.f_svg_print	= svg_print_net_icmp6_stats,
-	.hdr_line	= "imsg6/s;omsg6/s;iech6/s;iechr6/s;oechr6/s;igmbq6/s;igmbr6/s;ogmbr6/s;"
-			  "igmbrd6/s;ogmbrd6/s;irtsol6/s;ortsol6/s;irtad6/s;inbsol6/s;onbsol6/s;"
-			  "inbad6/s;onbad6/s",
 	.name		= "A_NET_ICMP6",
 	.g_nr		= 5,
 #endif
@@ -1013,13 +1065,15 @@ struct activity net_eicmp6_act = {
 	.f_print	= print_net_eicmp6_stats,
 	.f_print_avg	= print_net_eicmp6_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "ierr6/s;idtunr6/s;odtunr6/s;itmex6/s;otmex6/s;"
+		          "iprmpb6/s;oprmpb6/s;iredir6/s;oredir6/s;ipck2b6/s;opck2b6/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_eicmp6_stats,
 	.f_xml_print	= xml_print_net_eicmp6_stats,
 	.f_json_print	= json_print_net_eicmp6_stats,
 	.f_svg_print	= svg_print_net_eicmp6_stats,
-	.hdr_line	= "ierr6/s;idtunr6/s;odtunr6/s;itmex6/s;otmex6/s;"
-		          "iprmpb6/s;oprmpb6/s;iredir6/s;oredir6/s;ipck2b6/s;opck2b6/s",
 	.name		= "A_NET_EICMP6",
 	.g_nr		= 6,
 #endif
@@ -1048,12 +1102,14 @@ struct activity net_udp6_act = {
 	.f_print	= print_net_udp6_stats,
 	.f_print_avg	= print_net_udp6_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "idgm6/s;odgm6/s;noport6/s;idgmer6/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_net_udp6_stats,
 	.f_xml_print	= xml_print_net_udp6_stats,
 	.f_json_print	= json_print_net_udp6_stats,
 	.f_svg_print	= svg_print_net_udp6_stats,
-	.hdr_line	= "idgm6/s;odgm6/s;noport6/s;idgmer6/s",
 	.name		= "A_NET_UDP6",
 	.g_nr		= 2,
 #endif
@@ -1082,12 +1138,14 @@ struct activity pwr_cpufreq_act = {
 	.f_print	= print_pwr_cpufreq_stats,
 	.f_print_avg	= print_avg_pwr_cpufreq_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "CPU;MHz",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_cpufreq_stats,
 	.f_xml_print	= xml_print_pwr_cpufreq_stats,
 	.f_json_print	= json_print_pwr_cpufreq_stats,
 	.f_svg_print	= svg_print_pwr_cpufreq_stats,
-	.hdr_line	= "CPU;MHz",
 	.name		= "A_PWR_CPUFREQ",
 	.g_nr		= 1,
 #endif
@@ -1116,12 +1174,14 @@ struct activity pwr_fan_act = {
 	.f_print	= print_pwr_fan_stats,
 	.f_print_avg	= print_avg_pwr_fan_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "FAN;DEVICE;rpm;drpm",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_fan_stats,
 	.f_xml_print	= xml_print_pwr_fan_stats,
 	.f_json_print	= json_print_pwr_fan_stats,
 	.f_svg_print	= svg_print_pwr_fan_stats,
-	.hdr_line	= "FAN;DEVICE;rpm;drpm",
 	.name		= "A_PWR_FAN",
 	.g_nr		= 1,
 #endif
@@ -1150,12 +1210,14 @@ struct activity pwr_temp_act = {
 	.f_print	= print_pwr_temp_stats,
 	.f_print_avg	= print_avg_pwr_temp_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "TEMP;DEVICE;degC;%temp",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_temp_stats,
 	.f_xml_print	= xml_print_pwr_temp_stats,
 	.f_json_print	= json_print_pwr_temp_stats,
 	.f_svg_print	= svg_print_pwr_temp_stats,
-	.hdr_line	= "TEMP;DEVICE;degC;%temp",
 	.name		= "A_PWR_TEMP",
 	.g_nr		= 2,
 #endif
@@ -1184,12 +1246,14 @@ struct activity pwr_in_act = {
 	.f_print	= print_pwr_in_stats,
 	.f_print_avg	= print_avg_pwr_in_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "IN;DEVICE;inV;%in",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_in_stats,
 	.f_xml_print	= xml_print_pwr_in_stats,
 	.f_json_print	= json_print_pwr_in_stats,
 	.f_svg_print	= svg_print_pwr_in_stats,
-	.hdr_line	= "IN;DEVICE;inV;%in",
 	.name		= "A_PWR_IN",
 	.g_nr		= 2,
 #endif
@@ -1218,12 +1282,14 @@ struct activity huge_act = {
 	.f_print	= print_huge_stats,
 	.f_print_avg	= print_avg_huge_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "kbhugfree;kbhugused;%hugused",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_huge_stats,
 	.f_xml_print	= xml_print_huge_stats,
 	.f_json_print	= json_print_huge_stats,
 	.f_svg_print	= svg_print_huge_stats,
-	.hdr_line	= "kbhugfree;kbhugused;%hugused",
 	.name		= "A_HUGE",
 	.g_nr		= 2,
 #endif
@@ -1252,12 +1318,13 @@ struct activity pwr_wghfreq_act = {
 	.f_print	= print_pwr_wghfreq_stats,
 	.f_print_avg	= print_pwr_wghfreq_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "CPU;wghMHz",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_wghfreq_stats,
 	.f_xml_print	= xml_print_pwr_wghfreq_stats,
 	.f_json_print	= json_print_pwr_wghfreq_stats,
-	.f_svg_print	= NULL,
-	.hdr_line	= "CPU;wghMHz",
 	.name		= "A_PWR_WGHFREQ",
 	.g_nr		= 0,
 #endif
@@ -1286,12 +1353,13 @@ struct activity pwr_usb_act = {
 	.f_print	= print_pwr_usb_stats,
 	.f_print_avg	= print_avg_pwr_usb_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "manufact;product;BUS;idvendor;idprod;maxpower",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_usb_stats,
 	.f_xml_print	= xml_print_pwr_usb_stats,
 	.f_json_print	= json_print_pwr_usb_stats,
-	.f_svg_print	= NULL,
-	.hdr_line	= "manufact;product;BUS;idvendor;idprod;maxpower",
 	.name		= "A_PWR_USB",
 	.g_nr		= 0,
 #endif
@@ -1320,13 +1388,15 @@ struct activity filesystem_act = {
 	.f_print	= print_filesystem_stats,
 	.f_print_avg	= print_avg_filesystem_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "FILESYSTEM;MBfsfree;MBfsused;%fsused;%ufsused;Ifree;Iused;%Iused|"
+			  "MOUNTPOINT;MBfsfree;MBfsused;%fsused;%ufsused;Ifree;Iused;%Iused",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_filesystem_stats,
 	.f_xml_print	= xml_print_filesystem_stats,
 	.f_json_print	= json_print_filesystem_stats,
 	.f_svg_print	= svg_print_filesystem_stats,
-	.hdr_line	= "FILESYSTEM;MBfsfree;MBfsused;%fsused;%ufsused;Ifree;Iused;%Iused|"
-			  "MOUNTPOINT;MBfsfree;MBfsused;%fsused;%ufsused;Ifree;Iused;%Iused",
 	.name		= "A_FILESYSTEM",
 	.g_nr		= 4,
 #endif
@@ -1343,7 +1413,7 @@ struct activity filesystem_act = {
 /* Fibre Channel HBA usage activity */
 struct activity fchost_act = {
 	.id		= A_NET_FC,
-	.options	= AO_CLOSE_MARKUP + AO_GRAPH_PER_ITEM,
+	.options	= AO_GRAPH_PER_ITEM,
 	.magic		= ACTIVITY_MAGIC_BASE,
 	.group		= G_DISK,
 #ifdef SOURCE_SADC
@@ -1355,12 +1425,14 @@ struct activity fchost_act = {
 	.f_print	= print_fchost_stats,
 	.f_print_avg	= print_fchost_stats,
 #endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "FCHOST;fch_rxf/s;fch_txf/s;fch_rxw/s;fch_txw/s",
+#endif
 #ifdef SOURCE_SADF
 	.f_render	= render_fchost_stats,
 	.f_xml_print	= xml_print_fchost_stats,
 	.f_json_print	= json_print_fchost_stats,
 	.f_svg_print	= svg_print_fchost_stats,
-	.hdr_line	= "FCHOST;fch_rxf/s;fch_txf/s;fch_rxw/s;fch_txw/s",
 	.name		= "A_FCHOST",
 	.g_nr		= 2,
 #endif
@@ -1374,6 +1446,42 @@ struct activity fchost_act = {
 	.bitmap		= NULL
 };
 
+/* Softnet activity */
+struct activity softnet_act = {
+	.id		= A_NET_SOFT,
+	.options	= AO_COLLECTED + AO_CLOSE_MARKUP + AO_GRAPH_PER_ITEM,
+	.magic		= ACTIVITY_MAGIC_BASE,
+	.group		= G_DEFAULT,
+#ifdef SOURCE_SADC
+	.f_count_index	= 0,	/* wrap_get_cpu_nr() */
+	.f_count2	= NULL,
+	.f_read		= wrap_read_softnet,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_softnet_stats,
+	.f_print_avg	= print_softnet_stats,
+#endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "CPU;total/s;dropd/s;squeezd/s;rx_rps/s;flw_lim/s",
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_softnet_stats,
+	.f_xml_print	= xml_print_softnet_stats,
+	.f_json_print	= json_print_softnet_stats,
+	.f_svg_print	= svg_print_softnet_stats,
+	.name		= "A_NET_SOFT",
+	.g_nr		= 2,
+#endif
+	.nr		= -1,
+	.nr2		= 1,
+	.nr_max		= NR_CPUS + 1,
+	.fsize		= STATS_SOFTNET_SIZE,
+	.msize		= STATS_SOFTNET_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= &cpu_bitmap
+};
+
 #ifdef SOURCE_SADC
 /*
  * Array of functions used to count number of items.
@@ -1429,7 +1537,8 @@ struct activity *act[NR_ACT] = {
 	&net_icmp6_act,
 	&net_eicmp6_act,
 	&net_udp6_act,
-	&fchost_act,		/* AO_CLOSE_MARKUP */
+	&fchost_act,
+	&softnet_act,	/* AO_CLOSE_MARKUP */
 	/* </network> */
 	/* <power-management> */
 	&pwr_cpufreq_act,
