@@ -348,6 +348,69 @@ int is_iso_time_fmt(void)
 	return is_iso;
 }
 
+/*
+ ***************************************************************************
+ * Print tabulations
+ *
+ * IN:
+ * @nr_tab	Number of tabs to print.
+ ***************************************************************************
+ */
+void prtab(int nr_tab)
+{
+	int i;
+
+	for (i = 0; i < nr_tab; i++) {
+		printf("\t");
+	}
+}
+
+/*
+ ***************************************************************************
+ * printf() function modified for XML-like output. Don't print a CR at the
+ * end of the line.
+ *
+ * IN:
+ * @nr_tab	Number of tabs to print.
+ * @fmtf	printf() format.
+ ***************************************************************************
+ */
+void xprintf0(int nr_tab, const char *fmtf, ...)
+{
+	static char buf[1024];
+	va_list args;
+
+	va_start(args, fmtf);
+	vsnprintf(buf, sizeof(buf), fmtf, args);
+	va_end(args);
+
+	prtab(nr_tab);
+	printf("%s", buf);
+}
+
+/*
+ ***************************************************************************
+ * printf() function modified for XML-like output. Print a CR at the end of
+ * the line.
+ *
+ * IN:
+ * @nr_tab	Number of tabs to print.
+ * @fmtf	printf() format.
+ ***************************************************************************
+ */
+void xprintf(int nr_tab, const char *fmtf, ...)
+{
+	static char buf[1024];
+	va_list args;
+
+	va_start(args, fmtf);
+	vsnprintf(buf, sizeof(buf), fmtf, args);
+	va_end(args);
+
+	prtab(nr_tab);
+	printf("%s\n", buf);
+}
+
 /*
  ***************************************************************************
  * Print banner.
@@ -359,13 +422,15 @@ int is_iso_time_fmt(void)
  * @nodename	Hostname to display.
  * @machine	Machine architecture to display.
  * @cpu_nr	Number of CPU.
+ * @format	Set to FALSE for (default) plain output, and to TRUE for
+ * 		JSON format output.
  *
  * RETURNS:
  * TRUE if S_TIME_FORMAT is set to ISO, or FALSE otherwise.
  ***************************************************************************
  */
 int print_gal_header(struct tm *rectime, char *sysname, char *release,
-		     char *nodename, char *machine, int cpu_nr)
+		     char *nodename, char *machine, int cpu_nr, int format)
 {
 	char cur_date[TIMESTAMP_LEN];
 	int rc = 0;
@@ -381,8 +446,24 @@ int print_gal_header(struct tm *rectime, char *sysname, char *release,
 		strftime(cur_date, sizeof(cur_date), "%x", rectime);
 	}
 
-	printf("%s %s (%s) \t%s \t_%s_\t(%d CPU)\n", sysname, release, nodename,
-	       cur_date, machine, cpu_nr);
+	if (format == PLAIN_OUTPUT) {
+		/* Plain output */
+		printf("%s %s (%s) \t%s \t_%s_\t(%d CPU)\n", sysname, release, nodename,
+		       cur_date, machine, cpu_nr);
+	}
+	else {
+		/* JSON output */
+		xprintf(0, "{\"sysstat\": {");
+		xprintf(1, "\"hosts\": [");
+		xprintf(2, "{");
+		xprintf(3, "\"nodename\": \"%s\",", nodename);
+		xprintf(3, "\"sysname\": \"%s\",", sysname);
+		xprintf(3, "\"release\": \"%s\",", release);
+		xprintf(3, "\"machine\": \"%s\",", machine);
+		xprintf(3, "\"number-of-cpus\": %d,", cpu_nr);
+		xprintf(3, "\"date\": \"%s\",", cur_date);
+		xprintf(3, "\"statistics\": [");
+	}
 
 	return rc;
 }
@@ -1026,16 +1107,16 @@ void init_colors(void)
  *
  * IN:
  * @num		Number of values to print.
- * @wi		Output width.
+ * @width	Output width.
  ***************************************************************************
 */
-void cprintf_u64(int num, int wi, ...)
+void cprintf_u64(int num, int width, ...)
 {
 	int i;
 	uint64_t val;
 	va_list args;
 
-	va_start(args, wi);
+	va_start(args, width);
 
 	for (i = 0; i < num; i++) {
 		val = va_arg(args, unsigned long long);
@@ -1045,7 +1126,7 @@ void cprintf_u64(int num, int wi, ...)
 		else {
 			printf("%s", sc_int_stat);
 		}
-		printf(" %*"PRIu64, wi, val);
+		printf(" %*"PRIu64, width, val);
 		printf("%s", sc_normal);
 	}
 
@@ -1058,21 +1139,21 @@ void cprintf_u64(int num, int wi, ...)
  *
  * IN:
  * @num		Number of values to print.
- * @wi		Output width.
+ * @width	Output width.
  ***************************************************************************
 */
-void cprintf_x(int num, int wi, ...)
+void cprintf_x(int num, int width, ...)
 {
 	int i;
 	unsigned int val;
 	va_list args;
 
-	va_start(args, wi);
+	va_start(args, width);
 
 	for (i = 0; i < num; i++) {
 		val = va_arg(args, unsigned int);
 		printf("%s", sc_int_stat);
-		printf(" %*x", wi, val);
+		printf(" %*x", width, val);
 		printf("%s", sc_normal);
 	}
 
@@ -1085,7 +1166,7 @@ void cprintf_x(int num, int wi, ...)
  *
  * IN:
  * @num		Number of values to print.
- * @wi		Output width.
+ * @width	Output width.
  * @wd		Number of decimal places.
  ***************************************************************************
 */
@@ -1119,7 +1200,7 @@ void cprintf_f(int num, int wi, int wd, ...)
  *
  * IN:
  * @num		Number of values to print.
- * @wi		Output width.
+ * @width	Output width.
  * @wd		Number of decimal places.
  ***************************************************************************
 */
