@@ -29,6 +29,8 @@
 #define FALSE	0
 #define TRUE	1
 
+#define PLAIN_OUTPUT	0
+
 #define DISP_HDR	1
 
 /* Timestamp buffer length */
@@ -54,6 +56,7 @@
 #define K_ISO	"ISO"
 #define K_ALL	"ALL"
 #define K_UTC	"UTC"
+#define K_JSON	"JSON"
 
 /* Files */
 #define STAT			"/proc/stat"
@@ -268,12 +271,16 @@ double ll_sp_value
 int is_iso_time_fmt
 	(void);
 int print_gal_header
-	(struct tm *, char *, char *, char *, char *, int);
+	(struct tm *, char *, char *, char *, char *, int, int);
 void print_version
 	(void);
 char *strtolower
 	(char *);
 void sysstat_panic
 	(const char *, int);
+void xprintf
+	(int, const char *, ...);
+void xprintf0
+	(int, const char *, ...);
 
 #endif  /* _COMMON_H */
