@@ -692,13 +692,11 @@ __print_funct_t json_print_disk_stats(struct activity *a, int curr, int tab,
 				      unsigned long long itv)
 {
 	int i, j;
-	struct stats_disk *sdc,	*sdp, sdpzero;
+	struct stats_disk *sdc,	*sdp;
 	struct ext_disk_stats xds;
 	int sep = FALSE;
 	char *dev_name, *persist_dev_name;
 
-	memset(&sdpzero, 0, STATS_DISK_SIZE);
-
 	xprintf(tab++, "\"disk\": [");
 
 	for (i = 0; i < a->nr; i++) {
@@ -709,13 +707,7 @@ __print_funct_t json_print_disk_stats(struct activity *a, int curr, int tab,
 			continue;
 
 		j = check_disk_reg(a, curr, !curr, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			sdp = &sdpzero;
-		}
-		else {
-			sdp = (struct stats_disk *) ((char *) a->buf[!curr] + j * a->msize);
-		}
+		sdp = (struct stats_disk *) ((char *) a->buf[!curr] + j * a->msize);
 
 		/* Compute extended statistics values */
 		compute_ext_disk_stats(sdc, sdp, itv, &xds);
@@ -787,12 +779,10 @@ __print_funct_t json_print_net_dev_stats(struct activity *a, int curr, int tab,
 					 unsigned long long itv)
 {
 	int i, j;
-	struct stats_net_dev *sndc, *sndp, sndzero;
+	struct stats_net_dev *sndc, *sndp;
 	int sep = FALSE;
 	double rxkb, txkb, ifutil;
 
-	memset(&sndzero, 0, STATS_NET_DEV_SIZE);
-
 	if (!IS_SELECTED(a->options) || (a->nr <= 0))
 		goto close_json_markup;
 
@@ -806,16 +796,10 @@ __print_funct_t json_print_net_dev_stats(struct activity *a, int curr, int tab,
 		sndc = (struct stats_net_dev *) ((char *) a->buf[curr] + i * a->msize);
 
 		if (!strcmp(sndc->interface, ""))
-			break;
+			continue;
 
 		j = check_net_dev_reg(a, curr, !curr, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			sndp = &sndzero;
-		}
-		else {
-			sndp = (struct stats_net_dev *) ((char *) a->buf[!curr] + j * a->msize);
-		}
+		sndp = (struct stats_net_dev *) ((char *) a->buf[!curr] + j * a->msize);
 
 		if (sep) {
 			printf(",\n");
@@ -872,14 +856,12 @@ __print_funct_t json_print_net_edev_stats(struct activity *a, int curr, int tab,
 					  unsigned long long itv)
 {
 	int i, j;
-	struct stats_net_edev *snedc, *snedp, snedzero;
+	struct stats_net_edev *snedc, *snedp;
 	int sep = FALSE;
 
 	if (!IS_SELECTED(a->options) || (a->nr <= 0))
 		goto close_json_markup;
 
-	memset(&snedzero, 0, STATS_NET_EDEV_SIZE);
-
 	json_markup_network(tab, OPEN_JSON_MARKUP);
 	tab++;
 
@@ -890,16 +872,10 @@ __print_funct_t json_print_net_edev_stats(struct activity *a, int curr, int tab,
 		snedc = (struct stats_net_edev *) ((char *) a->buf[curr] + i * a->msize);
 
 		if (!strcmp(snedc->interface, ""))
-			break;
+			continue;
 
 		j = check_net_edev_reg(a, curr, !curr, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			snedp = &snedzero;
-		}
-		else {
-			snedp = (struct stats_net_edev *) ((char *) a->buf[!curr] + j * a->msize);
-		}
+		snedp = (struct stats_net_edev *) ((char *) a->buf[!curr] + j * a->msize);
 
 		if (sep) {
 			printf(",\n");
@@ -2257,3 +2233,94 @@ close_json_markup:
 		json_markup_network(tab, CLOSE_JSON_MARKUP);
 	}
 }
+
+/*
+ ***************************************************************************
+ * Display softnet statistics in JSON.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ * @tab		Indentation in output.
+ * @itv		Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t json_print_softnet_stats(struct activity *a, int curr, int tab,
+					 unsigned long long itv)
+{
+	int i;
+	struct stats_softnet *ssnc, *ssnp;
+	int sep = FALSE;
+	char cpuno[8];
+
+	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+		goto close_json_markup;
+
+	json_markup_network(tab, OPEN_JSON_MARKUP);
+	tab++;
+
+	xprintf(tab++, "\"softnet\": [");
+
+	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
+
+		/*
+		 * The size of a->buf[...] CPU structure may be different from the default
+		 * sizeof(struct stats_pwr_cpufreq) value if data have been read from a file!
+		 * That's why we don't use a syntax like:
+		 * ssnc = (struct stats_softnet *) a->buf[...] + i;
+                 */
+                ssnc = (struct stats_softnet *) ((char *) a->buf[curr] + i * a->msize);
+                ssnp = (struct stats_softnet *) ((char *) a->buf[!curr] + i * a->msize);
+
+		/*
+		 * Note: a->nr is in [1, NR_CPUS + 1].
+		 * Bitmap size is provided for (NR_CPUS + 1) CPUs.
+		 * Anyway, NR_CPUS may vary between the version of sysstat
+		 * used by sadc to create a file, and the version of sysstat
+		 * used by sar to read it...
+		 */
+
+		/* Should current CPU (including CPU "all") be displayed? */
+		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+			/* No */
+			continue;
+
+		/* Yes: Display current CPU stats */
+
+		if (sep) {
+			printf(",\n");
+		}
+		sep = TRUE;
+
+		if (!i) {
+			/* This is CPU "all" */
+			strcpy(cpuno, "all");
+		}
+		else {
+			sprintf(cpuno, "%d", i - 1);
+		}
+
+		xprintf0(tab, "{\"cpu\": \"%s\", "
+			 "\"total\": %.2f, "
+			 "\"dropd\": %.2f, "
+			 "\"squeezd\": %.2f, "
+			 "\"rx_rps\": %.2f, "
+			 "\"flw_lim\": %.2f}",
+			 cpuno,
+			 S_VALUE(ssnp->processed,    ssnc->processed,    itv),
+			 S_VALUE(ssnp->dropped,      ssnc->dropped,      itv),
+			 S_VALUE(ssnp->time_squeeze, ssnc->time_squeeze, itv),
+			 S_VALUE(ssnp->received_rps, ssnc->received_rps, itv),
+			 S_VALUE(ssnp->flow_limit,   ssnc->flow_limit,   itv));
+	}
+
+	printf("\n");
+	xprintf0(--tab, "]");
+
+	tab --;
+
+close_json_markup:
+	if (CLOSE_MARKUP(a->options)) {
+		json_markup_network(tab, CLOSE_JSON_MARKUP);
+	}
+}
