@@ -42,6 +42,70 @@ extern int  dis;
 extern char timestamp[][TIMESTAMP_LEN];
 extern unsigned long avg_count;
 
+
+/*
+ ***************************************************************************
+ * Display current activity header line.
+ *
+ * IN:
+ * @timestamp	Timestamp for previous stat sample.
+ * @a		Activity structure.
+ * @pos		Index in @.hdr_line string, 0 being the first one (header
+ * 		are delimited by the '|' character).
+ * @iwidth	First column width (generally this is the item name). A
+ * 		negative value means that the corresponding field shall be
+ * 		displayed at the end of the line, with no indication of width.
+ * @vwidth	Column width for stats values.
+ ***************************************************************************
+ */
+void print_hdr_line(char *timestamp, struct activity *a, int pos, int iwidth, int vwidth)
+{
+	char hline[HEADER_LINE_LEN] = "";
+	char *hl, *tk, *it = NULL;
+	int i = -1, j;
+	int p = pos;
+
+	strncpy(hline, a->hdr_line, HEADER_LINE_LEN - 1);
+	hline[HEADER_LINE_LEN - 1] = '\0';
+	for (hl = strtok(hline, "|"); hl && (pos > 0); hl = strtok(NULL, "|"), pos--);
+	if (!hl)
+		/* Bad @pos arg given to function */
+		return;
+
+	printf("\n%-11s", timestamp);
+
+	if (strchr(hl, '&')) {
+		j = strcspn(hl, "&");
+		if ((a->opt_flags & 0xff00) & (1 << (8 + p))) {
+			/* Display whole header line */
+			*(hl + j) = ';';
+		}
+		else {
+			/* Display only the first part of the header line */
+			*(hl + j) = '\0';
+		}
+	}
+	/* Display each field */
+	for (tk = strtok(hl, ";"); tk; tk = strtok(NULL, ";"), i--) {
+		if (iwidth > 0) {
+			printf(" %*s", iwidth, tk);
+			iwidth = 0;
+			continue;
+		}
+		if ((iwidth < 0) && (iwidth == i)) {
+			it = tk;
+			iwidth = 0;
+		}
+		else {
+			printf(" %*s", vwidth, tk);
+		}
+	}
+	if (it) {
+		printf(" %s", it);
+	}
+	printf("\n");
+}
+
 /*
  ***************************************************************************
  * Display CPU statistics.
@@ -61,17 +125,7 @@ __print_funct_t print_cpu_stats(struct activity *a, int prev, int curr,
 	struct stats_cpu *scc, *scp;
 
 	if (dis) {
-		if (DISPLAY_CPU_DEF(a->opt_flags)) {
-			printf("\n%-11s     CPU     %%user     %%nice   %%system"
-			       "   %%iowait    %%steal     %%idle\n",
-			       timestamp[!curr]);
-		}
-		else if (DISPLAY_CPU_ALL(a->opt_flags)) {
-			printf("\n%-11s     CPU      %%usr     %%nice      %%sys"
-			       "   %%iowait    %%steal      %%irq     %%soft"
-			       "    %%guest    %%gnice     %%idle\n",
-			       timestamp[!curr]);
-		}
+		print_hdr_line(timestamp[!curr], a, FIRST + DISPLAY_CPU_ALL(a->opt_flags), 7, 9);
 	}
 
 	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
@@ -94,117 +148,116 @@ __print_funct_t print_cpu_stats(struct activity *a, int prev, int curr,
 		 */
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
-			/* No */
-			continue;
+		if (a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) {
 
-		/* Yes: Display it */
-		printf("%-11s", timestamp[curr]);
+			/* Yes: Display it */
+			printf("%-11s", timestamp[curr]);
 
-		if (!i) {
-			/* This is CPU "all" */
-			cprintf_in(IS_STR, " %s", "    all", 0);
-		}
-		else {
-			cprintf_in(IS_INT, " %7d", "", i - 1);
+			if (!i) {
+				/* This is CPU "all" */
+				cprintf_in(IS_STR, " %s", "    all", 0);
+			}
+			else {
+				cprintf_in(IS_INT, " %7d", "", i - 1);
 
-			/*
-			 * If the CPU is offline then it is omited from /proc/stat:
-			 * All the fields couldn't have been read and the sum of them is zero.
-			 * (Remember that guest/guest_nice times are already included in
-			 * user/nice modes.)
-			 */
-			if ((scc->cpu_user    + scc->cpu_nice + scc->cpu_sys   +
-			     scc->cpu_iowait  + scc->cpu_idle + scc->cpu_steal +
-			     scc->cpu_hardirq + scc->cpu_softirq) == 0) {
 				/*
-				 * Set current struct fields (which have been set to zero)
-				 * to values from previous iteration. Hence their values won't
-				 * jump from zero when the CPU comes back online.
+				 * If the CPU is offline then it is omited from /proc/stat:
+				 * All the fields couldn't have been read and the sum of them is zero.
+				 * (Remember that guest/guest_nice times are already included in
+				 * user/nice modes.)
 				 */
-				*scc = *scp;
+				if ((scc->cpu_user    + scc->cpu_nice + scc->cpu_sys   +
+				     scc->cpu_iowait  + scc->cpu_idle + scc->cpu_steal +
+				     scc->cpu_hardirq + scc->cpu_softirq) == 0) {
+					/*
+					 * Set current struct fields (which have been set to zero)
+					 * to values from previous iteration. Hence their values won't
+					 * jump from zero when the CPU comes back online.
+					 */
+					*scc = *scp;
 
-				/* %user, %nice, %system, %iowait, %steal, ..., %idle */
-				cprintf_pc(6, 9, 2,
-					   0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
+					/* %user, %nice, %system, %iowait, %steal, ..., %idle */
+					cprintf_pc(6, 9, 2,
+						   0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
 
-				if (DISPLAY_CPU_ALL(a->opt_flags)) {
+					if (DISPLAY_CPU_ALL(a->opt_flags)) {
+						/*
+						 * Four additional fields to display:
+						 * %irq, %soft, %guest, %gnice.
+						 */
+						cprintf_pc(4, 9, 2,
+							   0.0, 0.0, 0.0, 0.0);
+					}
+					printf("\n");
+					continue;
+				}
+
+				/* Recalculate interval for current proc */
+				g_itv = get_per_cpu_interval(scc, scp);
+
+				if (!g_itv) {
+					/*
+					 * If the CPU is tickless then there is no change in CPU values
+					 * but the sum of values is not zero.
+					 * %user, %nice, %system, %iowait, %steal, ..., %idle
+					 */
+					cprintf_pc(5, 9, 2,
+						   0.0, 0.0, 0.0, 0.0, 0.0);
+
+					if (DISPLAY_CPU_DEF(a->opt_flags)) {
+						cprintf_pc(1, 9, 2, 100.0);
+						printf("\n");
+					}
 					/*
 					 * Four additional fields to display:
 					 * %irq, %soft, %guest, %gnice.
 					 */
-					cprintf_pc(4, 9, 2,
-						   0.0, 0.0, 0.0, 0.0);
+					else if (DISPLAY_CPU_ALL(a->opt_flags)) {
+						cprintf_pc(4, 9, 2,
+							   0.0, 0.0, 0.0, 100.0);
+						printf("\n");
+					}
+					continue;
 				}
+			}
+
+			if (DISPLAY_CPU_DEF(a->opt_flags)) {
+				cprintf_pc(6, 9, 2,
+					   ll_sp_value(scp->cpu_user, scc->cpu_user, g_itv),
+					   ll_sp_value(scp->cpu_nice, scc->cpu_nice, g_itv),
+					   ll_sp_value(scp->cpu_sys + scp->cpu_hardirq + scp->cpu_softirq,
+						       scc->cpu_sys + scc->cpu_hardirq + scc->cpu_softirq,
+						       g_itv),
+					   ll_sp_value(scp->cpu_iowait, scc->cpu_iowait, g_itv),
+					   ll_sp_value(scp->cpu_steal, scc->cpu_steal, g_itv),
+					   scc->cpu_idle < scp->cpu_idle ?
+					   0.0 :
+					   ll_sp_value(scp->cpu_idle, scc->cpu_idle, g_itv));
 				printf("\n");
-				continue;
 			}
-
-			/* Recalculate interval for current proc */
-			g_itv = get_per_cpu_interval(scc, scp);
-
-			if (!g_itv) {
-				/*
-				 * If the CPU is tickless then there is no change in CPU values
-				 * but the sum of values is not zero.
-				 * %user, %nice, %system, %iowait, %steal, ..., %idle
-				 */
-				cprintf_pc(5, 9, 2,
-					   0.0, 0.0, 0.0, 0.0, 0.0);
-
-				if (DISPLAY_CPU_DEF(a->opt_flags)) {
-					cprintf_pc(1, 9, 2, 100.0);
-					printf("\n");
-				}
-				/*
-				 * Four additional fields to display:
-				 * %irq, %soft, %guest, %gnice.
-				 */
-				else if (DISPLAY_CPU_ALL(a->opt_flags)) {
-					cprintf_pc(4, 9, 2,
-						   0.0, 0.0, 0.0, 100.0);
-					printf("\n");
-				}
-				continue;
+			else if (DISPLAY_CPU_ALL(a->opt_flags)) {
+				cprintf_pc(10, 9, 2,
+					   (scc->cpu_user - scc->cpu_guest) < (scp->cpu_user - scp->cpu_guest) ?
+					   0.0 :
+					   ll_sp_value(scp->cpu_user - scp->cpu_guest,
+						       scc->cpu_user - scc->cpu_guest, g_itv),
+						       (scc->cpu_nice - scc->cpu_guest_nice) < (scp->cpu_nice - scp->cpu_guest_nice) ?
+					   0.0 :
+					   ll_sp_value(scp->cpu_nice - scp->cpu_guest_nice,
+						       scc->cpu_nice - scc->cpu_guest_nice, g_itv),
+					   ll_sp_value(scp->cpu_sys, scc->cpu_sys, g_itv),
+					   ll_sp_value(scp->cpu_iowait, scc->cpu_iowait, g_itv),
+					   ll_sp_value(scp->cpu_steal, scc->cpu_steal, g_itv),
+					   ll_sp_value(scp->cpu_hardirq, scc->cpu_hardirq, g_itv),
+					   ll_sp_value(scp->cpu_softirq, scc->cpu_softirq, g_itv),
+					   ll_sp_value(scp->cpu_guest, scc->cpu_guest, g_itv),
+					   ll_sp_value(scp->cpu_guest_nice, scc->cpu_guest_nice, g_itv),
+					   scc->cpu_idle < scp->cpu_idle ?
+					   0.0 :
+					   ll_sp_value(scp->cpu_idle, scc->cpu_idle, g_itv));
+				printf("\n");
 			}
 		}
-
-		if (DISPLAY_CPU_DEF(a->opt_flags)) {
-			cprintf_pc(6, 9, 2,
-				   ll_sp_value(scp->cpu_user, scc->cpu_user, g_itv),
-				   ll_sp_value(scp->cpu_nice, scc->cpu_nice, g_itv),
-				   ll_sp_value(scp->cpu_sys + scp->cpu_hardirq + scp->cpu_softirq,
-					       scc->cpu_sys + scc->cpu_hardirq + scc->cpu_softirq,
-					       g_itv),
-				   ll_sp_value(scp->cpu_iowait, scc->cpu_iowait, g_itv),
-				   ll_sp_value(scp->cpu_steal, scc->cpu_steal, g_itv),
-				   scc->cpu_idle < scp->cpu_idle ?
-				   0.0 :
-				   ll_sp_value(scp->cpu_idle, scc->cpu_idle, g_itv));
-			printf("\n");
-		}
-		else if (DISPLAY_CPU_ALL(a->opt_flags)) {
-			cprintf_pc(10, 9, 2,
-				   (scc->cpu_user - scc->cpu_guest) < (scp->cpu_user - scp->cpu_guest) ?
-				   0.0 :
-				   ll_sp_value(scp->cpu_user - scp->cpu_guest,
-					       scc->cpu_user - scc->cpu_guest, g_itv),
-					       (scc->cpu_nice - scc->cpu_guest_nice) < (scp->cpu_nice - scp->cpu_guest_nice) ?
-				   0.0 :
-				   ll_sp_value(scp->cpu_nice - scp->cpu_guest_nice,
-					       scc->cpu_nice - scc->cpu_guest_nice, g_itv),
-				   ll_sp_value(scp->cpu_sys, scc->cpu_sys, g_itv),
-				   ll_sp_value(scp->cpu_iowait, scc->cpu_iowait, g_itv),
-				   ll_sp_value(scp->cpu_steal, scc->cpu_steal, g_itv),
-				   ll_sp_value(scp->cpu_hardirq, scc->cpu_hardirq, g_itv),
-				   ll_sp_value(scp->cpu_softirq, scc->cpu_softirq, g_itv),
-				   ll_sp_value(scp->cpu_guest, scc->cpu_guest, g_itv),
-				   ll_sp_value(scp->cpu_guest_nice, scc->cpu_guest_nice, g_itv),
-				   scc->cpu_idle < scp->cpu_idle ?
-				   0.0 :
-				   ll_sp_value(scp->cpu_idle, scc->cpu_idle, g_itv));
-			printf("\n");
-		}
 	}
 }
 
@@ -227,7 +280,7 @@ __print_funct_t print_pcsw_stats(struct activity *a, int prev, int curr,
 		*spp = (struct stats_pcsw *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s    proc/s   cswch/s\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -255,7 +308,7 @@ __print_funct_t print_irq_stats(struct activity *a, int prev, int curr,
 	struct stats_irq *sic, *sip;
 
 	if (dis) {
-		printf("\n%-11s      INTR    intr/s\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
@@ -309,7 +362,7 @@ __print_funct_t print_swap_stats(struct activity *a, int prev, int curr,
 		*ssp = (struct stats_swap *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s  pswpin/s pswpout/s\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -338,9 +391,7 @@ __print_funct_t print_paging_stats(struct activity *a, int prev, int curr,
 		*spp = (struct stats_paging *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s  pgpgin/s pgpgout/s   fault/s  majflt/s  pgfree/s"
-		       " pgscank/s pgscand/s pgsteal/s    %%vmeff\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -382,8 +433,7 @@ __print_funct_t print_io_stats(struct activity *a, int prev, int curr,
 		*sip = (struct stats_io *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s       tps      rtps      wtps   bread/s   bwrtn/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -446,8 +496,7 @@ void stub_print_memory_stats(struct activity *a, int prev, int curr,
 
 	if (DISPLAY_MEMORY(a->opt_flags)) {
 		if (dis) {
-			printf("\n%-11s   frmpg/s   bufpg/s   campg/s\n",
-			       timestamp[!curr]);
+			print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 		}
 
 		printf("%-11s", timestamp[curr]);
@@ -460,12 +509,7 @@ void stub_print_memory_stats(struct activity *a, int prev, int curr,
 
 	if (DISPLAY_MEM_AMT(a->opt_flags)) {
 		if (dis) {
-			printf("\n%-11s kbmemfree kbmemused  %%memused kbbuffers  kbcached"
-			       "  kbcommit   %%commit  kbactive   kbinact   kbdirty", timestamp[!curr]);
-			if (DISPLAY_MEM_ALL(a->opt_flags)) {
-				printf("  kbanonpg    kbslab  kbkstack   kbpgtbl  kbvmused");
-			}
-			printf("\n");
+			print_hdr_line(timestamp[!curr], a, SECOND, 0, 9);
 		}
 
 		if (!dispavg) {
@@ -565,8 +609,7 @@ void stub_print_memory_stats(struct activity *a, int prev, int curr,
 
 	if (DISPLAY_SWAP(a->opt_flags)) {
 		if (dis) {
-			printf("\n%-11s kbswpfree kbswpused  %%swpused  kbswpcad   %%swpcad\n",
-			       timestamp[!curr]);
+			print_hdr_line(timestamp[!curr], a, THIRD, 0, 9);
 		}
 
 		if (!dispavg) {
@@ -682,8 +725,7 @@ void stub_print_ktables_stats(struct activity *a, int curr, int dispavg)
 
 
 	if (dis) {
-		printf("\n%-11s dentunusd   file-nr  inode-nr    pty-nr\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	if (!dispavg) {
@@ -778,8 +820,7 @@ void stub_print_queue_stats(struct activity *a, int curr, int dispavg)
 		avg_procs_blocked = 0;
 
 	if (dis) {
-		printf("\n%-11s   runq-sz  plist-sz   ldavg-1   ldavg-5  ldavg-15   blocked\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	if (!dispavg) {
@@ -877,8 +918,7 @@ __print_funct_t print_serial_stats(struct activity *a, int prev, int curr,
 	struct stats_serial *ssc, *ssp;
 
 	if (dis) {
-		printf("\n%-11s       TTY   rcvin/s   xmtin/s framerr/s prtyerr/s"
-		       "     brk/s   ovrun/s\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	for (i = 0; i < a->nr; i++) {
@@ -924,16 +964,12 @@ __print_funct_t print_disk_stats(struct activity *a, int prev, int curr,
 				 unsigned long long itv)
 {
 	int i, j;
-	struct stats_disk *sdc,	*sdp, sdpzero;
+	struct stats_disk *sdc,	*sdp;
 	struct ext_disk_stats xds;
 	char *dev_name, *persist_dev_name;
 
-	memset(&sdpzero, 0, STATS_DISK_SIZE);
-
 	if (dis) {
-		printf("\n%-11s       DEV       tps  rd_sec/s  wr_sec/s  avgrq-sz"
-		       "  avgqu-sz     await     svctm     %%util\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	for (i = 0; i < a->nr; i++) {
@@ -944,13 +980,7 @@ __print_funct_t print_disk_stats(struct activity *a, int prev, int curr,
 			continue;
 
 		j = check_disk_reg(a, curr, prev, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			sdp = &sdpzero;
-		}
-		else {
-			sdp = (struct stats_disk *) ((char *) a->buf[prev] + j * a->msize);
-		}
+		sdp = (struct stats_disk *) ((char *) a->buf[prev] + j * a->msize);
 
 		/* Compute service time, etc. */
 		compute_ext_disk_stats(sdc, sdp, itv, &xds);
@@ -1009,14 +1039,11 @@ __print_funct_t print_net_dev_stats(struct activity *a, int prev, int curr,
 				    unsigned long long itv)
 {
 	int i, j;
-	struct stats_net_dev *sndc, *sndp, sndzero;
+	struct stats_net_dev *sndc, *sndp;
 	double rxkb, txkb, ifutil;
 
-	memset(&sndzero, 0, STATS_NET_DEV_SIZE);
-
 	if (dis) {
-		printf("\n%-11s     IFACE   rxpck/s   txpck/s    rxkB/s    txkB/s"
-		       "   rxcmp/s   txcmp/s  rxmcst/s   %%ifutil\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	for (i = 0; i < a->nr; i++) {
@@ -1024,16 +1051,10 @@ __print_funct_t print_net_dev_stats(struct activity *a, int prev, int curr,
 		sndc = (struct stats_net_dev *) ((char *) a->buf[curr] + i * a->msize);
 
 		if (!strcmp(sndc->interface, ""))
-			break;
+			continue;
 
 		j = check_net_dev_reg(a, curr, prev, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			sndp = &sndzero;
-		}
-		else {
-			sndp = (struct stats_net_dev *) ((char *) a->buf[prev] + j * a->msize);
-		}
+		sndp = (struct stats_net_dev *) ((char *) a->buf[prev] + j * a->msize);
 
 		printf("%-11s", timestamp[curr]);
 		cprintf_in(IS_STR, " %9s", sndc->interface, 0);
@@ -1070,14 +1091,10 @@ __print_funct_t print_net_edev_stats(struct activity *a, int prev, int curr,
 				     unsigned long long itv)
 {
 	int i, j;
-	struct stats_net_edev *snedc, *snedp, snedzero;
-
-	memset(&snedzero, 0, STATS_NET_EDEV_SIZE);
+	struct stats_net_edev *snedc, *snedp;
 
 	if (dis) {
-		printf("\n%-11s     IFACE   rxerr/s   txerr/s    coll/s  rxdrop/s"
-		       "  txdrop/s  txcarr/s  rxfram/s  rxfifo/s  txfifo/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	for (i = 0; i < a->nr; i++) {
@@ -1085,16 +1102,10 @@ __print_funct_t print_net_edev_stats(struct activity *a, int prev, int curr,
 		snedc = (struct stats_net_edev *) ((char *) a->buf[curr] + i * a->msize);
 
 		if (!strcmp(snedc->interface, ""))
-			break;
+			continue;
 
 		j = check_net_edev_reg(a, curr, prev, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			snedp = &snedzero;
-		}
-		else {
-			snedp = (struct stats_net_edev *) ((char *) a->buf[prev] + j * a->msize);
-		}
+		snedp = (struct stats_net_edev *) ((char *) a->buf[prev] + j * a->msize);
 
 		printf("%-11s", timestamp[curr]);
 		cprintf_in(IS_STR, " %9s", snedc->interface, 0);
@@ -1132,8 +1143,7 @@ __print_funct_t print_net_nfs_stats(struct activity *a, int prev, int curr,
 		*snnp = (struct stats_net_nfs *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s    call/s retrans/s    read/s   write/s  access/s"
-		       "  getatt/s\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1166,9 +1176,7 @@ __print_funct_t print_net_nfsd_stats(struct activity *a, int prev, int curr,
 		*snndp = (struct stats_net_nfsd *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s   scall/s badcall/s  packet/s     udp/s     tcp/s     "
-		       "hit/s    miss/s   sread/s  swrite/s saccess/s sgetatt/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1211,8 +1219,7 @@ void stub_print_net_sock_stats(struct activity *a, int curr, int dispavg)
 		avg_tcp_tw     = 0;
 
 	if (dis) {
-		printf("\n%-11s    totsck    tcpsck    udpsck    rawsck   ip-frag    tcp-tw\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	if (!dispavg) {
@@ -1306,8 +1313,7 @@ __print_funct_t print_net_ip_stats(struct activity *a, int prev, int curr,
 		*snip = (struct stats_net_ip *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s    irec/s  fwddgm/s    idel/s     orq/s   asmrq/s"
-		       "   asmok/s  fragok/s fragcrt/s\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1342,8 +1348,7 @@ __print_funct_t print_net_eip_stats(struct activity *a, int prev, int curr,
 		*sneip = (struct stats_net_eip *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s ihdrerr/s iadrerr/s iukwnpr/s   idisc/s   odisc/s"
-		       "   onort/s    asmf/s   fragf/s\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1378,9 +1383,7 @@ __print_funct_t print_net_icmp_stats(struct activity *a, int prev, int curr,
 		*snip = (struct stats_net_icmp *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s    imsg/s    omsg/s    iech/s   iechr/s    oech/s"
-		       "   oechr/s     itm/s    itmr/s     otm/s    otmr/s"
-		       "  iadrmk/s iadrmkr/s  oadrmk/s oadrmkr/s\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1421,9 +1424,7 @@ __print_funct_t print_net_eicmp_stats(struct activity *a, int prev, int curr,
 		*sneip = (struct stats_net_eicmp *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s    ierr/s    oerr/s idstunr/s odstunr/s   itmex/s"
-		       "   otmex/s iparmpb/s oparmpb/s   isrcq/s   osrcq/s"
-		       "  iredir/s  oredir/s\n", timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1462,8 +1463,7 @@ __print_funct_t print_net_tcp_stats(struct activity *a, int prev, int curr,
 		*sntp = (struct stats_net_tcp *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s  active/s passive/s    iseg/s    oseg/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1494,8 +1494,7 @@ __print_funct_t print_net_etcp_stats(struct activity *a, int prev, int curr,
 		*snetp = (struct stats_net_etcp *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s  atmptf/s  estres/s retrans/s isegerr/s   orsts/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1527,8 +1526,7 @@ __print_funct_t print_net_udp_stats(struct activity *a, int prev, int curr,
 		*snup = (struct stats_net_udp *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s    idgm/s    odgm/s  noport/s idgmerr/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1562,8 +1560,7 @@ void stub_print_net_sock6_stats(struct activity *a, int curr, int dispavg)
 		avg_frag6_inuse = 0;
 
 	if (dis) {
-		printf("\n%-11s   tcp6sck   udp6sck   raw6sck  ip6-frag\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	if (!dispavg) {
@@ -1650,9 +1647,7 @@ __print_funct_t print_net_ip6_stats(struct activity *a, int prev, int curr,
 		*snip = (struct stats_net_ip6 *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s   irec6/s fwddgm6/s   idel6/s    orq6/s  asmrq6/s"
-		       "  asmok6/s imcpck6/s omcpck6/s fragok6/s fragcr6/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1689,9 +1684,7 @@ __print_funct_t print_net_eip6_stats(struct activity *a, int prev, int curr,
 		*sneip = (struct stats_net_eip6 *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s ihdrer6/s iadrer6/s iukwnp6/s  i2big6/s  idisc6/s  odisc6/s"
-		       "  inort6/s  onort6/s   asmf6/s  fragf6/s itrpck6/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1729,10 +1722,7 @@ __print_funct_t print_net_icmp6_stats(struct activity *a, int prev, int curr,
 		*snip = (struct stats_net_icmp6 *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s   imsg6/s   omsg6/s   iech6/s  iechr6/s  oechr6/s"
-		       "  igmbq6/s  igmbr6/s  ogmbr6/s igmbrd6/s ogmbrd6/s irtsol6/s ortsol6/s"
-		       "  irtad6/s inbsol6/s onbsol6/s  inbad6/s  onbad6/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1776,9 +1766,7 @@ __print_funct_t print_net_eicmp6_stats(struct activity *a, int prev, int curr,
 		*sneip = (struct stats_net_eicmp6 *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s   ierr6/s idtunr6/s odtunr6/s  itmex6/s  otmex6/s"
-		       " iprmpb6/s oprmpb6/s iredir6/s oredir6/s ipck2b6/s opck2b6/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1816,8 +1804,7 @@ __print_funct_t print_net_udp6_stats(struct activity *a, int prev, int curr,
 		*snup = (struct stats_net_udp6 *) a->buf[prev];
 
 	if (dis) {
-		printf("\n%-11s   idgm6/s   odgm6/s noport6/s idgmer6/s\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	printf("%-11s", timestamp[curr]);
@@ -1858,8 +1845,7 @@ void stub_print_pwr_cpufreq_stats(struct activity *a, int curr, int dispavg)
 	}
 
 	if (dis) {
-		printf("\n%-11s     CPU       MHz\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 7, 9);
 	}
 
 	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
@@ -1971,7 +1957,6 @@ __print_funct_t print_avg_pwr_cpufreq_stats(struct activity *a, int prev, int cu
 void stub_print_pwr_fan_stats(struct activity *a, int curr, int dispavg)
 {
 	int i;
-	char fmt[16];
 	struct stats_pwr_fan *spc;
 	static double *avg_fan = NULL;
 	static double *avg_fan_min = NULL;
@@ -1993,15 +1978,14 @@ void stub_print_pwr_fan_stats(struct activity *a, int curr, int dispavg)
 	}
 
 	if (dis) {
-		printf("\n%-11s     FAN       rpm      drpm %*s\n",
-		       timestamp[!curr], MAX_SENSORS_DEV_LEN, "DEVICE");
+		print_hdr_line(timestamp[!curr], a, FIRST, -2, 9);
 	}
 
 	for (i = 0; i < a->nr; i++) {
 		spc = (struct stats_pwr_fan *) ((char *) a->buf[curr] + i * a->msize);
 
 		printf("%-11s", timestamp[curr]);
-		cprintf_in(IS_INT, "     %3d", "", i + 1);
+		cprintf_in(IS_INT, "     %5d", "", i + 1);
 
 		if (dispavg) {
 			/* Display average values */
@@ -2018,8 +2002,7 @@ void stub_print_pwr_fan_stats(struct activity *a, int curr, int dispavg)
 			avg_fan_min[i] += spc->rpm_min;
 		}
 
-		snprintf(fmt, 16, " %%%ds\n", MAX_SENSORS_DEV_LEN);
-		cprintf_in(IS_STR, fmt, spc->device, 0);
+		cprintf_in(IS_STR, " %s\n", spc->device, 0);
 	}
 
 	if (dispavg) {
@@ -2082,7 +2065,6 @@ __print_funct_t print_avg_pwr_fan_stats(struct activity *a, int prev, int curr,
 void stub_print_pwr_temp_stats(struct activity *a, int curr, int dispavg)
 {
 	int i;
-	char fmt[16];
 	struct stats_pwr_temp *spc;
 	static double *avg_temp = NULL;
 	static double *avg_temp_min = NULL, *avg_temp_max = NULL;
@@ -2111,15 +2093,14 @@ void stub_print_pwr_temp_stats(struct activity *a, int curr, int dispavg)
 	}
 
 	if (dis) {
-		printf("\n%-11s    TEMP      degC     %%temp %*s\n",
-		       timestamp[!curr], MAX_SENSORS_DEV_LEN, "DEVICE");
+		print_hdr_line(timestamp[!curr], a, FIRST, -2, 9);
 	}
 
 	for (i = 0; i < a->nr; i++) {
 		spc = (struct stats_pwr_temp *) ((char *) a->buf[curr] + i * a->msize);
 
 		printf("%-11s", timestamp[curr]);
-		cprintf_in(IS_INT, "     %3d", "", i + 1);
+		cprintf_in(IS_INT, "     %5d", "", i + 1);
 
 		if (dispavg) {
 			/* Display average values */
@@ -2142,8 +2123,7 @@ void stub_print_pwr_temp_stats(struct activity *a, int curr, int dispavg)
 			avg_temp_max[i] = spc->temp_max;
 		}
 
-		snprintf(fmt, 16, " %%%ds\n", MAX_SENSORS_DEV_LEN);
-		cprintf_in(IS_STR, fmt, spc->device, 0);
+		cprintf_in(IS_STR, " %s\n", spc->device, 0);
 	}
 
 	if (dispavg) {
@@ -2210,7 +2190,6 @@ __print_funct_t print_avg_pwr_temp_stats(struct activity *a, int prev, int curr,
 void stub_print_pwr_in_stats(struct activity *a, int curr, int dispavg)
 {
 	int i;
-	char fmt[16];
 	struct stats_pwr_in *spc;
 	static double *avg_in = NULL;
 	static double *avg_in_min = NULL, *avg_in_max = NULL;
@@ -2239,15 +2218,14 @@ void stub_print_pwr_in_stats(struct activity *a, int curr, int dispavg)
 	}
 
 	if (dis) {
-		printf("\n%-11s      IN       inV       %%in %*s\n",
-		       timestamp[!curr], MAX_SENSORS_DEV_LEN, "DEVICE");
+		print_hdr_line(timestamp[!curr], a, FIRST, -2, 9);
 	}
 
 	for (i = 0; i < a->nr; i++) {
 		spc = (struct stats_pwr_in *) ((char *) a->buf[curr] + i * a->msize);
 
 		printf("%-11s", timestamp[curr]);
-		cprintf_in(IS_INT, "     %3d", "", i);
+		cprintf_in(IS_INT, "     %5d", "", i);
 
 		if (dispavg) {
 			/* Display average values */
@@ -2270,8 +2248,7 @@ void stub_print_pwr_in_stats(struct activity *a, int curr, int dispavg)
 			avg_in_max[i] = spc->in_max;
 		}
 
-		snprintf(fmt, 16, " %%%ds\n", MAX_SENSORS_DEV_LEN);
-		cprintf_in(IS_STR, fmt, spc->device, 0);
+		cprintf_in(IS_STR, " %s\n", spc->device, 0);
 	}
 
 	if (dispavg) {
@@ -2344,8 +2321,7 @@ void stub_print_huge_stats(struct activity *a, int curr, int dispavg)
 		avg_tlhkb = 0;
 
 	if (dis) {
-		printf("\n%-11s kbhugfree kbhugused  %%hugused\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 0, 9);
 	}
 
 	if (!dispavg) {
@@ -2436,8 +2412,7 @@ void print_pwr_wghfreq_stats(struct activity *a, int prev, int curr,
 	unsigned long long tis, tisfreq;
 
 	if (dis) {
-		printf("\n%-11s     CPU    wghMHz\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, 7, 9);
 	}
 
 	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
@@ -2517,8 +2492,7 @@ void stub_print_pwr_usb_stats(struct activity *a, int curr, int dispavg)
 	if (dis) {
 		printf("\n%-11s     BUS  idvendor    idprod  maxpower",
 		       (dispavg ? _("Summary:") : timestamp[!curr]));
-		printf(" %*s", MAX_MANUF_LEN - 1, "manufact");
-		printf(" %*s\n", MAX_PROD_LEN - 1, "product");
+		printf(" %-*s product\n", MAX_MANUF_LEN - 1, "manufact");
 	}
 
 	for (i = 0; i < a->nr; i++) {
@@ -2537,10 +2511,9 @@ void stub_print_pwr_usb_stats(struct activity *a, int curr, int dispavg)
 			    /* bMaxPower is expressed in 2 mA units */
 			    (unsigned long long) (suc->bmaxpower << 1));
 
-		snprintf(fmt, 16, " %%%ds", MAX_MANUF_LEN - 1);
+		snprintf(fmt, 16, " %%-%ds", MAX_MANUF_LEN - 1);
 		cprintf_s(IS_STR, fmt, suc->manufacturer);
-		snprintf(fmt, 16, " %%%ds\n", MAX_PROD_LEN - 1);
-		cprintf_s(IS_STR, fmt, suc->product);
+		cprintf_s(IS_STR, " %s\n", suc->product);
 
 		if (!dispavg) {
 			/* Save current USB device in summary list */
@@ -2630,10 +2603,8 @@ __print_funct_t stub_print_filesystem_stats(struct activity *a, int curr, int di
 	struct stats_filesystem *sfc, *sfm;
 
 	if (dis) {
-		printf("\n%-11s  MBfsfree  MBfsused   %%fsused  %%ufsused"
-		       "     Ifree     Iused    %%Iused %s\n",
-		       (dispavg ? _("Summary:") : timestamp[!curr]),
-		       DISPLAY_MOUNT(a->opt_flags) ? "MOUNTPOINT" : "FILESYSTEM");
+		print_hdr_line((dispavg ? _("Summary:") : timestamp[!curr]),
+			       a, FIRST + DISPLAY_MOUNT(a->opt_flags), -1, 9);
 	}
 
 	for (i = 0; i < a->nr; i++) {
@@ -2733,8 +2704,7 @@ __print_funct_t print_fchost_stats(struct activity *a, int prev, int curr,
 	struct stats_fchost *sfcc,*sfcp;
 
 	if (dis) {
-		printf("\n%-11s fch_rxf/s fch_txf/s fch_rxw/s fch_txw/s FCHOST\n",
-		       timestamp[!curr]);
+		print_hdr_line(timestamp[!curr], a, FIRST, -1, 9);
 	}
 
 	for (i = 0; i < a->nr; i++) {
@@ -2755,3 +2725,70 @@ __print_funct_t print_fchost_stats(struct activity *a, int prev, int curr,
 		cprintf_in(IS_STR, " %s\n", sfcc->fchost_name, 0);
 	}
 }
+
+/*
+ ***************************************************************************
+ * Display softnet statistics.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @prev	Index in array where stats used as reference are.
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t print_softnet_stats(struct activity *a, int prev, int curr,
+				    unsigned long long itv)
+{
+	struct stats_softnet
+		*ssnc = (struct stats_softnet *) a->buf[curr],
+		*ssnp = (struct stats_softnet *) a->buf[prev];
+	int i;
+
+	if (dis) {
+		print_hdr_line(timestamp[!curr], a, FIRST, 7, 9);
+	}
+
+	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
+
+		/*
+		 * The size of a->buf[...] CPU structure may be different from the default
+		 * sizeof(struct stats_pwr_cpufreq) value if data have been read from a file!
+		 * That's why we don't use a syntax like:
+		 * ssnc = (struct stats_softnet *) a->buf[...] + i;
+                 */
+                ssnc = (struct stats_softnet *) ((char *) a->buf[curr] + i * a->msize);
+                ssnp = (struct stats_softnet *) ((char *) a->buf[prev] + i * a->msize);
+
+		/*
+		 * Note: a->nr is in [1, NR_CPUS + 1].
+		 * Bitmap size is provided for (NR_CPUS + 1) CPUs.
+		 * Anyway, NR_CPUS may vary between the version of sysstat
+		 * used by sadc to create a file, and the version of sysstat
+		 * used by sar to read it...
+		 */
+
+		/* Should current CPU (including CPU "all") be displayed? */
+		if (a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) {
+
+			/* Yes: Display it */
+			printf("%-11s", timestamp[curr]);
+
+			if (!i) {
+				/* This is CPU "all" */
+				cprintf_in(IS_STR, " %s", "    all", 0);
+			}
+			else {
+				cprintf_in(IS_INT, " %7d", "", i - 1);
+			}
+
+			cprintf_f(5, 9, 2,
+				  S_VALUE(ssnp->processed,    ssnc->processed,    itv),
+				  S_VALUE(ssnp->dropped,      ssnc->dropped,      itv),
+				  S_VALUE(ssnp->time_squeeze, ssnc->time_squeeze, itv),
+				  S_VALUE(ssnp->received_rps, ssnc->received_rps, itv),
+				  S_VALUE(ssnp->flow_limit,   ssnc->flow_limit,   itv));
+			printf("\n");
+		}
+	}
+}
