@@ -58,6 +58,7 @@
 #define NET_SOCKSTAT6	"/proc/net/sockstat6"
 #define NET_RPC_NFS	"/proc/net/rpc/nfs"
 #define NET_RPC_NFSD	"/proc/net/rpc/nfsd"
+#define NET_SOFTNET	"/proc/net/softnet_stat"
 #define LOADAVG		"/proc/loadavg"
 #define VMSTAT		"/proc/vmstat"
 #define NET_SNMP	"/proc/net/snmp"
@@ -568,6 +569,17 @@ struct stats_fchost {
 
 #define STATS_FCHOST_SIZE	(sizeof(struct stats_fchost))
 
+/* Structure for softnet statistics */
+struct stats_softnet {
+	unsigned int processed		__attribute__ ((aligned (4)));
+	unsigned int dropped		__attribute__ ((packed));
+	unsigned int time_squeeze	__attribute__ ((packed));
+	unsigned int received_rps	__attribute__ ((packed));
+	unsigned int flow_limit		__attribute__ ((packed));
+};
+
+#define STATS_SOFTNET_SIZE	(sizeof(struct stats_softnet))
+
 /*
  ***************************************************************************
  * Prototypes for functions used to read system statistics
@@ -650,5 +662,7 @@ void read_filesystem
 	(struct stats_filesystem *, int);
 void read_fchost
 	(struct stats_fchost *, int);
+void read_softnet
+	(struct stats_softnet *, int);
 
 #endif /* _RD_STATS_H */
