@@ -1047,14 +1047,12 @@ __print_funct_t render_disk_stats(struct activity *a, int isdb, char *pre,
 				  int curr, unsigned long long itv)
 {
 	int i, j;
-	struct stats_disk *sdc,	*sdp, sdpzero;
+	struct stats_disk *sdc,	*sdp;
 	struct ext_disk_stats xds;
 	char *dev_name, *persist_dev_name;
 	int pt_newlin
 		= (DISPLAY_HORIZONTALLY(flags) ? PT_NOFLAG : PT_NEWLIN);
 
-	memset(&sdpzero, 0, STATS_DISK_SIZE);
-
 	for (i = 0; i < a->nr; i++) {
 
 		sdc = (struct stats_disk *) ((char *) a->buf[curr] + i * a->msize);
@@ -1063,13 +1061,7 @@ __print_funct_t render_disk_stats(struct activity *a, int isdb, char *pre,
 			continue;
 
 		j = check_disk_reg(a, curr, !curr, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			sdp = &sdpzero;
-		}
-		else {
-			sdp = (struct stats_disk *) ((char *) a->buf[!curr] + j * a->msize);
-		}
+		sdp = (struct stats_disk *) ((char *) a->buf[!curr] + j * a->msize);
 
 		/* Compute extended stats (service time, etc.) */
 		compute_ext_disk_stats(sdc, sdp, itv, &xds);
@@ -1169,28 +1161,20 @@ __print_funct_t render_net_dev_stats(struct activity *a, int isdb, char *pre,
 				     int curr, unsigned long long itv)
 {
 	int i, j;
-	struct stats_net_dev *sndc, *sndp, sndzero;
+	struct stats_net_dev *sndc, *sndp;
 	double rxkb, txkb, ifutil;
 	int pt_newlin
 		= (DISPLAY_HORIZONTALLY(flags) ? PT_NOFLAG : PT_NEWLIN);
 
-	memset(&sndzero, 0, STATS_NET_DEV_SIZE);
-
 	for (i = 0; i < a->nr; i++) {
 
 		sndc = (struct stats_net_dev *) ((char *) a->buf[curr] + i * a->msize);
 
 		if (!strcmp(sndc->interface, ""))
-			break;
+			continue;
 
 		j = check_net_dev_reg(a, curr, !curr, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			sndp = &sndzero;
-		}
-		else {
-			sndp = (struct stats_net_dev *) ((char *) a->buf[!curr] + j * a->msize);
-		}
+		sndp = (struct stats_net_dev *) ((char *) a->buf[!curr] + j * a->msize);
 
 		render(isdb, pre, PT_NOFLAG,
 		       "%s\trxpck/s", "%s",
@@ -1269,27 +1253,19 @@ __print_funct_t render_net_edev_stats(struct activity *a, int isdb, char *pre,
 				      int curr, unsigned long long itv)
 {
 	int i, j;
-	struct stats_net_edev *snedc, *snedp, snedzero;
+	struct stats_net_edev *snedc, *snedp;
 	int pt_newlin
 		= (DISPLAY_HORIZONTALLY(flags) ? PT_NOFLAG : PT_NEWLIN);
 
-	memset(&snedzero, 0, STATS_NET_EDEV_SIZE);
-
 	for (i = 0; i < a->nr; i++) {
 
 		snedc = (struct stats_net_edev *) ((char *) a->buf[curr] + i * a->msize);
 
 		if (!strcmp(snedc->interface, ""))
-			break;
+			continue;
 
 		j = check_net_edev_reg(a, curr, !curr, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			snedp = &snedzero;
-		}
-		else {
-			snedp = (struct stats_net_edev *) ((char *) a->buf[!curr] + j * a->msize);
-		}
+		snedp = (struct stats_net_edev *) ((char *) a->buf[!curr] + j * a->msize);
 
 		render(isdb, pre, PT_NOFLAG,
 		       "%s\trxerr/s", "%s",
@@ -3008,3 +2984,122 @@ __print_funct_t render_fchost_stats(struct activity *a, int isdb, char *pre,
 		       NULL);
 	}
 }
+
+/*
+ ***************************************************************************
+ * Display softnet statistics in selected format.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @isdb	Flag, true if db printing, false if ppc printing.
+ * @pre		Prefix string for output entries
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t render_softnet_stats(struct activity *a, int isdb, char *pre,
+				     int curr, unsigned long long itv)
+{
+	int i;
+	struct stats_softnet *ssnc, *ssnp;
+	int pt_newlin
+		= (DISPLAY_HORIZONTALLY(flags) ? PT_NOFLAG : PT_NEWLIN);
+
+	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
+
+		/*
+		 * The size of a->buf[...] CPU structure may be different from the default
+		 * sizeof(struct stats_pwr_cpufreq) value if data have been read from a file!
+		 * That's why we don't use a syntax like:
+		 * ssnc = (struct stats_softnet *) a->buf[...] + i;
+                 */
+                ssnc = (struct stats_softnet *) ((char *) a->buf[curr] + i * a->msize);
+                ssnp = (struct stats_softnet *) ((char *) a->buf[!curr] + i * a->msize);
+
+		/*
+		 * Note: a->nr is in [1, NR_CPUS + 1].
+		 * Bitmap size is provided for (NR_CPUS + 1) CPUs.
+		 * Anyway, NR_CPUS may vary between the version of sysstat
+		 * used by sadc to create a file, and the version of sysstat
+		 * used by sar to read it...
+		 */
+
+		/* Should current CPU (including CPU "all") be displayed? */
+		if (a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) {
+
+			if (!i) {
+				/* This is CPU "all" */
+				render(isdb, pre, PT_NOFLAG,
+				       "all\ttotal/s",
+				       "-1", NULL,
+				       NOVAL,
+				       S_VALUE(ssnp->processed, ssnc->processed, itv),
+				       NULL);
+
+				render(isdb, pre, PT_NOFLAG,
+				       "all\tdropd/s",
+				       NULL, NULL,
+				       NOVAL,
+				       S_VALUE(ssnp->dropped, ssnc->dropped, itv),
+				       NULL);
+
+				render(isdb, pre, PT_NOFLAG,
+				       "all\tsqueezd/s",
+				       NULL, NULL,
+				       NOVAL,
+				       S_VALUE(ssnp->time_squeeze, ssnc->time_squeeze, itv),
+				       NULL);
+
+				render(isdb, pre, PT_NOFLAG,
+				       "all\trx_rps/s",
+				       NULL, NULL,
+				       NOVAL,
+				       S_VALUE(ssnp->received_rps, ssnc->received_rps, itv),
+				       NULL);
+
+				render(isdb, pre, pt_newlin,
+				       "all\tflw_lim/s",
+				       NULL, NULL,
+				       NOVAL,
+				       S_VALUE(ssnp->flow_limit, ssnc->flow_limit, itv),
+				       NULL);
+			}
+			else {
+				render(isdb, pre, PT_NOFLAG,
+				       "cpu%d\ttotal/s",
+				       "%d", cons(iv, i - 1, NOVAL),
+				       NOVAL,
+				       S_VALUE(ssnp->processed, ssnc->processed, itv),
+				       NULL);
+
+				render(isdb, pre, PT_NOFLAG,
+				       "cpu%d\tdropd/s",
+				       NULL, cons(iv, i - 1, NOVAL),
+				       NOVAL,
+				       S_VALUE(ssnp->dropped, ssnc->dropped, itv),
+				       NULL);
+
+				render(isdb, pre, PT_NOFLAG,
+				       "cpu%d\tsqueezd/s",
+				       NULL, cons(iv, i - 1, NOVAL),
+				       NOVAL,
+				       S_VALUE(ssnp->time_squeeze, ssnc->time_squeeze, itv),
+				       NULL);
+
+				render(isdb, pre, PT_NOFLAG,
+				       "cpu%d\trx_rps/s",
+				       NULL, cons(iv, i - 1, NOVAL),
+				       NOVAL,
+				       S_VALUE(ssnp->received_rps, ssnc->received_rps, itv),
+				       NULL);
+
+				render(isdb, pre, pt_newlin,
+				       "cpu%d\tflw_lim/s",
+				       NULL, cons(iv, i - 1, NOVAL),
+				       NOVAL,
+				       S_VALUE(ssnp->flow_limit, ssnc->flow_limit, itv),
+				       NULL);
+			}
+		}
+	}
+}
