@@ -20,7 +20,7 @@
  */
 
 /* Number of activities */
-#define NR_ACT		38
+#define NR_ACT		39
 /* The value below is used for sanity check */
 #define MAX_NR_ACT	256
 
@@ -66,6 +66,7 @@
 #define A_PWR_USB	36
 #define A_FILESYSTEM	37
 #define A_NET_FC	38
+#define A_NET_SOFT	39
 
 
 /* Macro used to flag an activity that should be collected */
@@ -187,6 +188,7 @@
 #define K_FREQ		"FREQ"
 #define K_MOUNT		"MOUNT"
 #define K_FC		"FC"
+#define K_SOFT		"SOFT"
 
 #define K_INT		"INT"
 #define K_DISK		"DISK"
@@ -266,6 +268,10 @@
 #define SOFT_SIZE	0
 #define HARD_SIZE	1
 
+#define FIRST	0
+#define SECOND	1
+#define THIRD	2
+
 #define CLOSE_XML_MARKUP	0
 #define OPEN_XML_MARKUP		1
 
@@ -1064,6 +1070,8 @@ __read_funct_t wrap_read_filesystem
 	(struct activity *);
 __read_funct_t wrap_read_fchost
 	(struct activity *);
+__read_funct_t wrap_read_softnet
+	(struct activity *);
 
 /* Other functions */
 void allocate_bitmaps
@@ -1077,10 +1085,10 @@ int check_disk_reg
 void check_file_actlst
 	(int *, char *, struct activity * [], struct file_magic *, struct file_header *,
 	 struct file_activity **, unsigned int [], int);
-int check_net_dev_reg
-	(struct activity *, int, int, int);
-int check_net_edev_reg
-	(struct activity *, int, int, int);
+unsigned int check_net_dev_reg
+	(struct activity *, int, int, unsigned int);
+unsigned int check_net_edev_reg
+	(struct activity *, int, int, unsigned int);
 double compute_ifutil
 	(struct stats_net_dev *, double, double);
 void copy_structures
