@@ -611,7 +611,8 @@ void print_report_hdr(unsigned int flags, struct tm *rectime,
 	/* Display the header */
 	print_gal_header(rectime, file_hdr->sa_sysname, file_hdr->sa_release,
 			 file_hdr->sa_nodename, file_hdr->sa_machine,
-			 cpu_nr > 1 ? cpu_nr - 1 : 1);
+			 cpu_nr > 1 ? cpu_nr - 1 : 1,
+			 PLAIN_OUTPUT);
 }
 
 /*
@@ -627,20 +628,14 @@ void print_report_hdr(unsigned int flags, struct tm *rectime,
  *
  * RETURNS:
  * Position of current network interface in array of sample statistics used
- * as reference, or -1 if it is a new interface (it was not present in array
- * of stats used as reference).
- *
- * Note: A newly registered interface, if it is supernumerary, may make the
- * last interface in the array going out of the list. Yet an interface going
- * out of the list still exists in the /proc/net/dev file. Should it go back
- * in the list (e.g. if some other interfaces have been unregistered) then
- * its counters will jump as if starting from zero.
+ * as reference.
  ***************************************************************************
  */
-int check_net_dev_reg(struct activity *a, int curr, int ref, int pos)
+unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
+			       unsigned int pos)
 {
 	struct stats_net_dev *sndc, *sndp;
-	int index = 0;
+	unsigned int index = 0;
 
 	sndc = (struct stats_net_dev *) ((char *) a->buf[curr] + pos * a->msize);
 
@@ -713,8 +708,23 @@ int check_net_dev_reg(struct activity *a, int curr, int ref, int pos)
 		index++;
 	}
 
-	/* This is a newly registered interface */
-	return -1;
+	/* Network interface not found: Look for the first free structure */
+	for (index = 0; index < a->nr; index++) {
+		sndp = (struct stats_net_dev *) ((char *) a->buf[ref] + index * a->msize);
+		if (!strcmp(sndp->interface, ""))
+			break;
+	}
+	if (index >= a->nr) {
+		/* No free structure: Default is structure of same rank */
+		index = pos;
+	}
+
+	sndp = (struct stats_net_dev *) ((char *) a->buf[ref] + index * a->msize);
+	/* Since the name is not the same, reset all the structure */
+	memset(sndp, 0, STATS_NET_DEV_SIZE);
+	strncpy(sndp->interface, sndc->interface, MAX_IFACE_LEN - 1);
+
+	return  index;
 }
 
 /*
@@ -730,13 +740,14 @@ int check_net_dev_reg(struct activity *a, int curr, int ref, int pos)
  *
  * RETURNS:
  * Position of current network interface in array of sample statistics used
- * as reference, or -1 if it is a newly registered interface.
+ * as reference.
  ***************************************************************************
  */
-int check_net_edev_reg(struct activity *a, int curr, int ref, int pos)
+unsigned int check_net_edev_reg(struct activity *a, int curr, int ref,
+				unsigned int pos)
 {
 	struct stats_net_edev *snedc, *snedp;
-	int index = 0;
+	unsigned int index = 0;
 
 	snedc = (struct stats_net_edev *) ((char *) a->buf[curr] + pos * a->msize);
 
@@ -769,8 +780,23 @@ int check_net_edev_reg(struct activity *a, int curr, int ref, int pos)
 		index++;
 	}
 
-	/* This is a newly registered interface */
-	return -1;
+	/* Network interface not found: Look for the first free structure */
+	for (index = 0; index < a->nr; index++) {
+		snedp = (struct stats_net_edev *) ((char *) a->buf[ref] + index * a->msize);
+		if (!strcmp(snedp->interface, ""))
+			break;
+	}
+	if (index >= a->nr) {
+		/* No free structure: Default is structure of same rank */
+		index = pos;
+	}
+
+	snedp = (struct stats_net_edev *) ((char *) a->buf[ref] + index * a->msize);
+	/* Since the name is not the same, reset all the structure */
+	memset(snedp, 0, STATS_NET_EDEV_SIZE);
+	strncpy(snedp->interface, snedc->interface, MAX_IFACE_LEN - 1);
+
+	return  index;
 }
 
 /*
@@ -785,8 +811,7 @@ int check_net_edev_reg(struct activity *a, int curr, int ref, int pos)
  * @pos		Index on current disk.
  *
  * RETURNS:
- * Position of current disk in array of sample statistics used as reference
- * or -1 if it is a newly registered device.
+ * Position of current disk in array of sample statistics used as reference.
  ***************************************************************************
  */
 int check_disk_reg(struct activity *a, int curr, int ref, int pos)
@@ -820,8 +845,24 @@ int check_disk_reg(struct activity *a, int curr, int ref, int pos)
 		index++;
 	}
 
-	/* This is a newly registered device */
-	return -1;
+	/* Disk not found: Look for the first free structure */
+	for (index = 0; index < a->nr; index++) {
+		sdp = (struct stats_disk *) ((char *) a->buf[ref] + index * a->msize);
+		if (!(sdp->major + sdp->minor))
+			break;
+	}
+	if (index >= a->nr) {
+		/* No free structure found: Default is structure of same rank */
+		index = pos;
+	}
+
+	sdp = (struct stats_disk *) ((char *) a->buf[ref] + index * a->msize);
+	/* Since the device is not the same, reset all the structure */
+	memset(sdp, 0, STATS_DISK_SIZE);
+	sdp->major = sdc->major;
+	sdp->minor = sdc->minor;
+
+	return index;
 }
 
 /*
@@ -1540,8 +1581,11 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 		case 'A':
 			select_all_activities(act);
 
-			/* Force '-P ALL -I XALL -r ALL -u ALL' */
-
+			/*
+			 * Force '-P ALL -I XALL -r ALL -u ALL -F'.
+			 * Setting -F is compulsory because corresponding activity
+			 * has AO_MULTIPLE_OUTPUTS flag set.
+			 */
 			p = get_activity_position(act, A_MEMORY, EXIT_IF_NOT_FOUND);
 			act[p]->opt_flags |= AO_F_MEM_AMT + AO_F_MEM_DIA +
 					     AO_F_MEM_SWAP + AO_F_MEM_ALL;
@@ -1834,6 +1878,9 @@ int parse_sar_n_opt(char *argv[], int *opt, struct activity *act[])
 		else if (!strcmp(t, K_FC)) {
 			SELECT_ACTIVITY(A_NET_FC);
 		}
+		else if (!strcmp(t, K_SOFT)) {
+			SELECT_ACTIVITY(A_NET_SOFT);
+		}
 		else if (!strcmp(t, K_ALL)) {
 			SELECT_ACTIVITY(A_NET_DEV);
 			SELECT_ACTIVITY(A_NET_EDEV);
@@ -1854,6 +1901,7 @@ int parse_sar_n_opt(char *argv[], int *opt, struct activity *act[])
 			SELECT_ACTIVITY(A_NET_EICMP6);
 			SELECT_ACTIVITY(A_NET_UDP6);
 			SELECT_ACTIVITY(A_NET_FC);
+			SELECT_ACTIVITY(A_NET_SOFT);
 		}
 		else
 			return 1;
