@@ -529,9 +529,11 @@ __tm_funct_t print_db_timestamp(void *parm, int action, char *cur_date,
  * @action	Action expected from current function.
  * @cur_date	Date string of current comment.
  * @cur_time	Time string of current comment.
+ * @utc		True if @cur_time is expressed in UTC.
  * @itv		Interval of time with preceding record.
- * @file_hdr	System activity file standard header (unused here).
- * @flags	Flags for common options.
+ *
+ * OUT:
+ * @tab		Number of tabulations.
  ***************************************************************************
  */
 __tm_funct_t print_xml_timestamp(void *parm, int action, char *cur_date,
@@ -561,9 +563,11 @@ __tm_funct_t print_xml_timestamp(void *parm, int action, char *cur_date,
  * @action	Action expected from current function.
  * @cur_date	Date string of current comment.
  * @cur_time	Time string of current comment.
+ * @utc		True if @cur_time is expressed in UTC.
  * @itv		Interval of time with preceding record.
- * @file_hdr	System activity file standard header (unused here).
- * @flags	Flags for common options.
+ *
+ * OUT:
+ * @tab		Number of tabulations.
  ***************************************************************************
  */
 __tm_funct_t print_json_timestamp(void *parm, int action, char *cur_date,
@@ -604,7 +608,7 @@ __tm_funct_t print_json_timestamp(void *parm, int action, char *cur_date,
  * @id_seq	Activity sequence (unused here).
  *
  * OUT:
- * @parm	Number of tabulations.
+ * @tab		Number of tabulations.
  ***************************************************************************
  */
 __printf_funct_t print_xml_header(void *parm, int action, char *dfile,
@@ -671,7 +675,7 @@ __printf_funct_t print_xml_header(void *parm, int action, char *dfile,
  * @id_seq	Activity sequence (unused here).
  *
  * OUT:
- * @parm	Number of tabulations.
+ * @tab		Number of tabulations.
  ***************************************************************************
  */
 __printf_funct_t print_json_header(void *parm, int action, char *dfile,
@@ -686,10 +690,7 @@ __printf_funct_t print_json_header(void *parm, int action, char *dfile,
 	if (action & F_BEGIN) {
 		xprintf(*tab, "{\"sysstat\": {");
 
-		xprintf(++(*tab), "\"sysdata-version\": %s,",
-			XML_DTD_VERSION);
-
-		xprintf(*tab, "\"hosts\": [");
+		xprintf(++(*tab), "\"hosts\": [");
 		xprintf(++(*tab), "{");
 		xprintf(++(*tab), "\"nodename\": \"%s\",", file_hdr->sa_nodename);
 		xprintf(*tab, "\"sysname\": \"%s\",", file_hdr->sa_sysname);
@@ -762,7 +763,8 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 		print_gal_header(localtime((const time_t *) &(file_hdr->sa_ust_time)),
 				 file_hdr->sa_sysname, file_hdr->sa_release,
 				 file_hdr->sa_nodename, file_hdr->sa_machine,
-				 cpu_nr > 1 ? cpu_nr - 1 : 1);
+				 cpu_nr > 1 ? cpu_nr - 1 : 1,
+				 PLAIN_OUTPUT);
 
 		printf(_("Number of CPU for last samples in file: %u\n"),
 		       file_hdr->sa_last_cpu_nr > 1 ? file_hdr->sa_last_cpu_nr - 1 : 1);
@@ -844,7 +846,8 @@ __printf_funct_t print_svg_header(void *parm, int action, char *dfile,
 		print_gal_header(localtime((const time_t *) &(file_hdr->sa_ust_time)),
 				 file_hdr->sa_sysname, file_hdr->sa_release,
 				 file_hdr->sa_nodename, file_hdr->sa_machine,
-				 cpu_nr > 1 ? cpu_nr - 1 : 1);
+				 cpu_nr > 1 ? cpu_nr - 1 : 1,
+				 PLAIN_OUTPUT);
 		printf("</text>\n");
 	}
 
