@@ -170,7 +170,8 @@ void display_help(char *progname)
 		 "\t\tICMP6\tICMP traffic\t(v6)\n"
 		 "\t\tEICMP6\tICMP traffic\t(v6) (errors)\n"
 		 "\t\tUDP6\tUDP traffic\t(v6)\n"
-		 "\t\tFC\tFibre channel HBAs\n"));
+		 "\t\tFC\tFibre channel HBAs\n"
+		 "\t\tSOFT\tSoftware-based network processing\n"));
 	printf(_("\t-q\tQueue length and load average statistics\n"));
 	printf(_("\t-R\tMemory statistics\n"));
 	printf(_("\t-r [ ALL ]\n"
