@@ -36,8 +36,14 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/utsname.h>
+
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #undef HZ /* sys/param.h defines HZ but needed for MAXPATHLEN */
+#endif
+#ifndef MAXPATHLEN
+#define MAXPATHLEN 256
+#endif
 
 #include "version.h"
 #include "tapestat.h"
@@ -686,7 +692,8 @@ int main(int argc, char **argv)
 	/* Get system name, release number and hostname */
 	uname(&header);
 	if (print_gal_header(&rectime, header.sysname, header.release,
-			     header.nodename, header.machine, cpu_nr)) {
+			     header.nodename, header.machine, cpu_nr,
+			     PLAIN_OUTPUT)) {
 		flags |= T_D_ISO;
 	}
 	printf("\n");
