@@ -674,12 +674,10 @@ __print_funct_t xml_print_disk_stats(struct activity *a, int curr, int tab,
 				     unsigned long long itv)
 {
 	int i, j;
-	struct stats_disk *sdc,	*sdp, sdpzero;
+	struct stats_disk *sdc,	*sdp;
 	struct ext_disk_stats xds;
 	char *dev_name, *persist_dev_name;
 
-	memset(&sdpzero, 0, STATS_DISK_SIZE);
-
 	xprintf(tab++, "<disk per=\"second\">");
 
 	for (i = 0; i < a->nr; i++) {
@@ -690,13 +688,7 @@ __print_funct_t xml_print_disk_stats(struct activity *a, int curr, int tab,
 			continue;
 
 		j = check_disk_reg(a, curr, !curr, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			sdp = &sdpzero;
-		}
-		else {
-			sdp = (struct stats_disk *) ((char *) a->buf[!curr] + j * a->msize);
-		}
+		sdp = (struct stats_disk *) ((char *) a->buf[!curr] + j * a->msize);
 
 		/* Compute extended statistics values */
 		compute_ext_disk_stats(sdc, sdp, itv, &xds);
@@ -762,14 +754,12 @@ __print_funct_t xml_print_net_dev_stats(struct activity *a, int curr, int tab,
 					unsigned long long itv)
 {
 	int i, j;
-	struct stats_net_dev *sndc, *sndp, sndzero;
+	struct stats_net_dev *sndc, *sndp;
 	double rxkb, txkb, ifutil;
 
 	if (!IS_SELECTED(a->options) || (a->nr <= 0))
 		goto close_xml_markup;
 
-	memset(&sndzero, 0, STATS_NET_DEV_SIZE);
-
 	xml_markup_network(tab, OPEN_XML_MARKUP);
 	tab++;
 
@@ -778,16 +768,10 @@ __print_funct_t xml_print_net_dev_stats(struct activity *a, int curr, int tab,
 		sndc = (struct stats_net_dev *) ((char *) a->buf[curr] + i * a->msize);
 
 		if (!strcmp(sndc->interface, ""))
-			break;
+			continue;
 
 		j = check_net_dev_reg(a, curr, !curr, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			sndp = &sndzero;
-		}
-		else {
-			sndp = (struct stats_net_dev *) ((char *) a->buf[!curr] + j * a->msize);
-		}
+		sndp = (struct stats_net_dev *) ((char *) a->buf[!curr] + j * a->msize);
 
 		rxkb = S_VALUE(sndp->rx_bytes, sndc->rx_bytes, itv);
 		txkb = S_VALUE(sndp->tx_bytes, sndc->tx_bytes, itv);
@@ -835,13 +819,11 @@ __print_funct_t xml_print_net_edev_stats(struct activity *a, int curr, int tab,
 					 unsigned long long itv)
 {
 	int i, j;
-	struct stats_net_edev *snedc, *snedp, snedzero;
+	struct stats_net_edev *snedc, *snedp;
 
 	if (!IS_SELECTED(a->options) || (a->nr <= 0))
 		goto close_xml_markup;
 
-	memset(&snedzero, 0, STATS_NET_EDEV_SIZE);
-
 	xml_markup_network(tab, OPEN_XML_MARKUP);
 	tab++;
 
@@ -850,16 +832,10 @@ __print_funct_t xml_print_net_edev_stats(struct activity *a, int curr, int tab,
 		snedc = (struct stats_net_edev *) ((char *) a->buf[curr] + i * a->msize);
 
 		if (!strcmp(snedc->interface, ""))
-			break;
+			continue;
 
 		j = check_net_edev_reg(a, curr, !curr, i);
-		if (j < 0) {
-			/* This is a newly registered interface. Previous stats are zero */
-			snedp = &snedzero;
-		}
-		else {
-			snedp = (struct stats_net_edev *) ((char *) a->buf[!curr] + j * a->msize);
-		}
+		snedp = (struct stats_net_edev *) ((char *) a->buf[!curr] + j * a->msize);
 
 		xprintf(tab, "<net-edev iface=\"%s\" "
 			"rxerr=\"%.2f\" "
@@ -2142,3 +2118,67 @@ close_xml_markup:
 		xml_markup_network(tab, CLOSE_XML_MARKUP);
 	}
 }
+
+/*
+ ***************************************************************************
+ * Display softnet statistics in XML.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ * @tab		Indentation in XML output.
+ * @itv		Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t xml_print_softnet_stats(struct activity *a, int curr, int tab,
+					unsigned long long itv)
+{
+	int i;
+	struct stats_softnet *ssnc, *ssnp;
+	char cpuno[8];
+
+	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+		goto close_xml_markup;
+
+	xml_markup_network(tab, OPEN_XML_MARKUP);
+	tab++;
+
+	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
+
+		ssnc = (struct stats_softnet *) ((char *) a->buf[curr]  + i * a->msize);
+		ssnp = (struct stats_softnet *) ((char *) a->buf[!curr] + i * a->msize);
+
+		/* Should current CPU (including CPU "all") be displayed? */
+		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+			/* No */
+			continue;
+
+		/* Yes: Display it */
+		if (!i) {
+			/* This is CPU "all" */
+			strcpy(cpuno, "all");
+		}
+		else {
+			sprintf(cpuno, "%d", i - 1);
+		}
+
+		xprintf(tab, "<softnet cpu=\"%s\" "
+			"total=\"%.2f\" "
+			"dropd=\"%.2f\" "
+			"squeezd=\"%.2f\" "
+			"rx_rps=\"%.2f\" "
+			"flw_lim=\"%.2f\"/>",
+			 cpuno,
+			 S_VALUE(ssnp->processed,    ssnc->processed,    itv),
+			 S_VALUE(ssnp->dropped,      ssnc->dropped,      itv),
+			 S_VALUE(ssnp->time_squeeze, ssnc->time_squeeze, itv),
+			 S_VALUE(ssnp->received_rps, ssnc->received_rps, itv),
+			 S_VALUE(ssnp->flow_limit,   ssnc->flow_limit,   itv));
+	}
+	tab--;
+
+close_xml_markup:
+	if (CLOSE_MARKUP(a->options)) {
+		xml_markup_network(tab, CLOSE_XML_MARKUP);
+	}
+}
